/***************************************************************************
*   Copyright (C) 2003 by Sandro Giessl                                   *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/

#include <kconfig.h>
#include <klocale.h>
#include <kpixmap.h>
#include <kpixmapeffect.h>

#include <qcursor.h>
#include <qfontmetrics.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qtooltip.h>

#include "alloyclient.h"
#include "misc.h"
#include "embeddata.h"

// Default button layout
const char default_left[]  = "M";
const char default_right[] = "HIAX";

// construct all areas which need more complex paintings only once at the
// beginning... hope this brings performance benefits
static QPixmap *active_button_border = 0;
static QPixmap *inactive_button_border = 0;

static QPixmap *active_button_common_normal = 0;
static QPixmap *active_button_common_hover = 0;
static QPixmap *active_button_common_sunken = 0;
static QPixmap *active_button_close_normal = 0;
static QPixmap *active_button_close_hover = 0;
static QPixmap *active_button_close_sunken = 0;

static QPixmap *inactive_button_common_normal = 0;
static QPixmap *inactive_button_common_hover = 0;
static QPixmap *inactive_button_common_sunken = 0;
static QPixmap *inactive_button_close_normal = 0;
static QPixmap *inactive_button_close_hover = 0;
static QPixmap *inactive_button_close_sunken = 0;

static QPixmap *active_titlebar_blank = 0;
static QPixmap *active_titlebar_grooves = 0;
static QPixmap *active_titlebar_left = 0;
static QPixmap *active_titlebar_right = 0;
static QPixmap *inactive_titlebar_blank = 0;
static QPixmap *inactive_titlebar_grooves = 0;
static QPixmap *inactive_titlebar_left = 0;
static QPixmap *inactive_titlebar_right = 0;

static QPixmap *active_resizer_detail_3 = 0;   // 5 px border
static QPixmap *inactive_resizer_detail_3 = 0; // 5 px border
static QPixmap *active_resizer_detail_2 = 0;   // 4 px border
static QPixmap *inactive_resizer_detail_2 = 0; // 4 px border

static bool pixmaps_created = false;

// global constants
static const int BUTTONSIZE      = 16; // do not change
static const int TITLESIZE       = 21; // do not change
static const int TITLESPACER1    = 8; // TITLESPACER1-stripes-TITLESPACER2-TITLE-
static const int TITLESPACER2    = 5; //      -TITLESPACER2-stripes-TITLESPACER1
static const int FRAMESIZE_TOP   = 1; // do not change
static const int BUTTONSPACING   = 2; // 2 px between buttons


bool AlloyHandler::m_initialized = false;
bool AlloyHandler::m_titleShadow = true;
int  AlloyHandler::m_borderSize = 5;

static void create_pixmaps()
{
    if(pixmaps_created)
        return;

    QColor aColor = KDecoration::options()->color(KDecoration::ColorTitleBar, true);
    QColor iColor = KDecoration::options()->color(KDecoration::ColorTitleBar, false);
    QColor aCloseColor = QColor(255,60,0);
    QColor iCloseColor = aCloseColor;

    QPainter painter;

    active_button_border = new QPixmap(BUTTONSIZE, BUTTONSIZE);
    painter.begin(active_button_border);
    drawButtonBorder(&painter, aColor);
    painter.end();
    inactive_button_border = new QPixmap(BUTTONSIZE, BUTTONSIZE);
    painter.begin(inactive_button_border);
    drawButtonBorder(&painter, iColor);
    painter.end();

    active_button_common_normal = new QPixmap(BUTTONSIZE-2, BUTTONSIZE-2);
    painter.begin(active_button_common_normal);
    drawButtonBase(&painter, aColor, aColor, 1);
    painter.end();
    inactive_button_common_normal = new QPixmap(BUTTONSIZE-2, BUTTONSIZE-2);
    painter.begin(inactive_button_common_normal);
    drawButtonBase(&painter, iColor, iColor, 1);
    painter.end();

    active_button_common_hover = new QPixmap(BUTTONSIZE-2, BUTTONSIZE-2);
    painter.begin(active_button_common_hover);
    drawButtonBase(&painter, aColor, aColor, 2);
    painter.end();
    inactive_button_common_hover = new QPixmap(BUTTONSIZE-2, BUTTONSIZE-2);
    painter.begin(inactive_button_common_hover);
    drawButtonBase(&painter, iColor, iColor, 2);
    painter.end();

    active_button_common_sunken = new QPixmap(BUTTONSIZE-2, BUTTONSIZE-2);
    painter.begin(active_button_common_sunken);
    drawButtonBase(&painter, aColor, aColor, 3);
    painter.end();
    inactive_button_common_sunken = new QPixmap(BUTTONSIZE-2, BUTTONSIZE-2);
    painter.begin(inactive_button_common_sunken);
    drawButtonBase(&painter, iColor, iColor, 3);
    painter.end();

    // close...
    active_button_close_normal = new QPixmap(BUTTONSIZE-2, BUTTONSIZE-2);
    painter.begin(active_button_close_normal);
    drawButtonBase(&painter, aCloseColor, aColor, 1);
    painter.end();
    inactive_button_close_normal = new QPixmap(BUTTONSIZE-2, BUTTONSIZE-2);
    painter.begin(inactive_button_close_normal);
    drawButtonBase(&painter, iCloseColor, iColor, 1);
    painter.end();

    active_button_close_hover = new QPixmap(BUTTONSIZE-2, BUTTONSIZE-2);
    painter.begin(active_button_close_hover);
    drawButtonBase(&painter, aCloseColor, aColor, 2);
    painter.end();
    inactive_button_close_hover = new QPixmap(BUTTONSIZE-2, BUTTONSIZE-2);
    painter.begin(inactive_button_close_hover);
    drawButtonBase(&painter, iCloseColor, iColor, 2);
    painter.end();

    active_button_close_sunken = new QPixmap(BUTTONSIZE-2, BUTTONSIZE-2);
    painter.begin(active_button_close_sunken);
    drawButtonBase(&painter, aCloseColor, aColor, 3);
    painter.end();
    inactive_button_close_sunken = new QPixmap(BUTTONSIZE-2, BUTTONSIZE-2);
    painter.begin(inactive_button_close_sunken);
    drawButtonBase(&painter, iCloseColor, iColor, 3);
    painter.end();


    KPixmap tempKPixmap;
    int i;

    active_titlebar_blank = new QPixmap(1, TITLESIZE-1);
    painter.begin(active_titlebar_blank);
    tempKPixmap.resize(1, TITLESIZE-1);
    KPixmapEffect::gradient(tempKPixmap,
                            hsvRelative(aColor, 0, -50, 32), // less saturation, brighter
                            aColor,
                            KPixmapEffect::VerticalGradient);
    painter.drawPixmap(0, 0, tempKPixmap);
    painter.end();

    inactive_titlebar_blank = new QPixmap(1, TITLESIZE-1);
    painter.begin(inactive_titlebar_blank);
    tempKPixmap.resize(1, TITLESIZE-1);
    KPixmapEffect::gradient(tempKPixmap,
                            hsvRelative(iColor, 0, -50, 32), // less saturation, brighter
                            iColor,
                            KPixmapEffect::VerticalGradient);
    painter.drawPixmap(0, 0, tempKPixmap);
    painter.end();


    active_titlebar_grooves = new QPixmap(1, TITLESIZE-1);
    painter.begin(active_titlebar_grooves);
    tempKPixmap.resize(1, TITLESIZE-1);
    KPixmapEffect::gradient(tempKPixmap,
                            hsvRelative(aColor, 0, -50, 32), // less saturation, brighter
                            aColor,
                            KPixmapEffect::VerticalGradient);
    painter.drawPixmap(0, 0, tempKPixmap);
    for(i = 1; i < 5; i++)
    {
        // s: -10; v: +10
        painter.setPen(hsvRelative(aColor, 0, -(50)/(TITLESIZE-1)*((TITLESIZE-1)-i*4) - 10, (32)/(TITLESIZE-1)*((TITLESIZE-1)-i*4) + 10) );
        painter.drawPoint(0, i*4 -1);
        // s: 0; v: -8
        painter.setPen(hsvRelative(aColor, 0, -(50)/(TITLESIZE-1)*((TITLESIZE-1)-i*4) + 0, (32)/(TITLESIZE-1)*((TITLESIZE-1)-i*4) -8) );
        painter.drawPoint(0, i*4+1 -1);
    }
    painter.end();

    inactive_titlebar_grooves = new QPixmap(1, TITLESIZE-1);
    painter.begin(inactive_titlebar_grooves);
    tempKPixmap.resize(1, TITLESIZE-1);
    KPixmapEffect::gradient(tempKPixmap,
                            hsvRelative(iColor, 0, -50, 32), // less saturation, brighter
                            iColor,
                            KPixmapEffect::VerticalGradient);
    painter.drawPixmap(0, 0, tempKPixmap);
    for(i = 1; i < 5; i++)
    {
        // s: -10; v: +10
        painter.setPen(hsvRelative(iColor, 0, -(50)/(TITLESIZE-1)*((TITLESIZE-1)-i*4) - 10, (32)/(TITLESIZE-1)*((TITLESIZE-1)-i*4) + 10) );
        painter.drawPoint(0, i*4 -1);
        // s: 0; v: -8
        painter.setPen(hsvRelative(iColor, 0, -(50)/(TITLESIZE-1)*((TITLESIZE-1)-i*4) + 0, (32)/(TITLESIZE-1)*((TITLESIZE-1)-i*4) -8) );
        painter.drawPoint(0, i*4+1 -1);
    }
    painter.end();


    active_titlebar_left = new QPixmap(1, TITLESIZE-1);
    painter.begin(active_titlebar_left);
    tempKPixmap.resize(1, TITLESIZE-1);
    KPixmapEffect::gradient(tempKPixmap,
                            hsvRelative(aColor, 0, -80, +40),
                            hsvRelative(aColor, 0, -50, +20),
                            KPixmapEffect::VerticalGradient);
    painter.drawPixmap(0, 0, tempKPixmap);
    painter.end();

    inactive_titlebar_left = new QPixmap(1, TITLESIZE-1);
    painter.begin(inactive_titlebar_left);
    tempKPixmap.resize(1, TITLESIZE-1);
    KPixmapEffect::gradient(tempKPixmap,
                            hsvRelative(iColor, 0, -80, +40),
                            hsvRelative(iColor, 0, -50, +20),
                            KPixmapEffect::VerticalGradient);
    painter.drawPixmap(0, 0, tempKPixmap);
    painter.end();


    active_titlebar_right = new QPixmap(1, TITLESIZE-1);
    painter.begin(active_titlebar_right);
    tempKPixmap.resize(1, TITLESIZE-1);
    KPixmapEffect::gradient(tempKPixmap,
                            hsvRelative(aColor, 0, -80, -50),
                            hsvRelative(aColor, 0, 0, -50),
                            KPixmapEffect::VerticalGradient);
    painter.drawPixmap(0, 0, tempKPixmap);
    painter.end();

    inactive_titlebar_right = new QPixmap(1, TITLESIZE-1);
    painter.begin(inactive_titlebar_right);
    tempKPixmap.resize(1, TITLESIZE-1);
    KPixmapEffect::gradient(tempKPixmap,
                            hsvRelative(iColor, 0, -80, -50),
                            hsvRelative(iColor, 0, 0, -50),
                            KPixmapEffect::VerticalGradient);
    painter.drawPixmap(0, 0, tempKPixmap);
    painter.end();

    active_resizer_detail_3 = new QPixmap(3, 3);
    painter.begin(active_resizer_detail_3);
    painter.setPen(hsvRelative(aColor, 0, 0, -50) );
    painter.drawLine(0, 0, 1, 0);
    painter.drawPoint(0, 1);
    painter.setPen(hsvRelative(aColor, 0, -50, +20) );
    painter.drawPoint(2, 1);
    painter.drawLine(1, 2, 2, 2);
    painter.setPen(aColor);
    painter.drawPoint(2, 0);
    painter.drawPoint(1, 1);
    painter.drawPoint(0, 2);
    painter.end();

    inactive_resizer_detail_3 = new QPixmap(3, 3);
    painter.begin(inactive_resizer_detail_3);
    painter.setPen(hsvRelative(iColor, 0, 0, -50) );
    painter.drawLine(0, 0, 1, 0);
    painter.drawPoint(0, 1);
    painter.setPen(hsvRelative(iColor, 0, -50, +20) );
    painter.drawPoint(2, 1);
    painter.drawLine(1, 2, 2, 2);
    painter.setPen(iColor);
    painter.drawPoint(2, 0);
    painter.drawPoint(1, 1);
    painter.drawPoint(0, 2);
    painter.end();

    active_resizer_detail_2 = new QPixmap(2, 2);
    painter.begin(active_resizer_detail_2);
    painter.setPen(hsvRelative(aColor, 0, 0, -50) );
    painter.drawLine(0, 0, 1, 0);
    painter.drawPoint(0, 1);
    painter.setPen(hsvRelative(aColor, 0, -50, +20) );
    painter.drawPoint(1, 1);
    painter.end();

    inactive_resizer_detail_2 = new QPixmap(2, 2);
    painter.begin(inactive_resizer_detail_2);
    painter.setPen(hsvRelative(iColor, 0, 0, -50) );
    painter.drawLine(0, 0, 1, 0);
    painter.drawPoint(0, 1);
    painter.setPen(hsvRelative(iColor, 0, -50, +20) );
    painter.drawPoint(1, 1);
    painter.end();
}

static void delete_pixmaps()
{
    delete active_button_border;
    active_button_border = 0;
    delete inactive_button_border;
    inactive_button_border = 0;

    delete active_button_common_normal;
    active_button_common_normal = 0;
    delete active_button_common_hover;
    active_button_common_hover = 0;
    delete active_button_common_sunken;
    active_button_common_sunken = 0;

    delete inactive_button_common_normal;
    inactive_button_common_normal = 0;
    delete inactive_button_common_hover;
    inactive_button_common_hover = 0;
    delete inactive_button_common_sunken;
    inactive_button_common_sunken = 0;

    delete active_button_close_normal;
    active_button_close_normal = 0;
    delete active_button_close_hover;
    active_button_close_hover = 0;
    delete active_button_close_sunken;
    active_button_close_sunken = 0;

    delete inactive_button_close_normal;
    inactive_button_close_normal = 0;
    delete inactive_button_close_hover;
    inactive_button_close_hover = 0;
    delete inactive_button_close_sunken;
    inactive_button_close_sunken = 0;

    delete active_titlebar_blank;
    active_titlebar_blank = 0;
    delete active_titlebar_grooves;
    active_titlebar_grooves = 0;
    delete active_titlebar_left;
    active_titlebar_left = 0;
    delete active_titlebar_right;
    active_titlebar_right = 0;

    delete inactive_titlebar_blank;
    inactive_titlebar_blank = 0;
    delete inactive_titlebar_grooves;
    inactive_titlebar_grooves = 0;
    delete inactive_titlebar_left;
    inactive_titlebar_left = 0;
    delete inactive_titlebar_right;
    inactive_titlebar_right = 0;

    delete active_resizer_detail_3;
    active_resizer_detail_3 = 0;
    delete inactive_resizer_detail_3;
    inactive_resizer_detail_3 = 0;
    delete active_resizer_detail_2;
    active_resizer_detail_2 = 0;
    delete inactive_resizer_detail_2;
    inactive_resizer_detail_2 = 0;

    pixmaps_created = false;
}

AlloyHandler::AlloyHandler()
{
    reset(0);
}

AlloyHandler::~AlloyHandler()
{
    delete_pixmaps();
    qCleanupImages_KWinAlloy();
    m_initialized = false;
}

bool AlloyHandler::reset( unsigned long changed )
{
    // read in the configuration
    readConfig();

    delete_pixmaps();
    create_pixmaps();
    qInitImages_KWinAlloy();

    // reset all clients
    m_initialized = true;
    // Do we need to "hit the wooden hammer" ?
    bool needHardReset = true;
//     if (changed & SettingColors || changed & SettingFont)
//     {
//         needHardReset = false;
//     }
    if (needHardReset)
        return true;
    else {
        resetDecorations(changed);
        return false;
    }
}

KDecoration* AlloyHandler::createDecoration( KDecorationBridge* bridge )
{
        return new AlloyClient( bridge, this );
}

void AlloyHandler::readConfig()
{
    // create a config object
    KConfig config("kwinalloyrc");
    config.setGroup("General");

    // grab settings
    m_titleShadow = config.readBoolEntry("TitleShadow", true);
    switch(KDecoration::options()->preferredBorderSize( this )) {
        case BorderTiny:
            m_borderSize = 3;
            break;
        case BorderLarge:
            m_borderSize = 9;
            break;
        case BorderVeryLarge:
            m_borderSize = 13;
            break;
        case BorderHuge:
            m_borderSize = 17;
            break;
        case BorderVeryHuge:
            m_borderSize = 27;
            break;
        case BorderOversized:
            m_borderSize = 35;
            break;
        case BorderNormal:
        default:
            m_borderSize = 5;
    }
//     int size = config.readNumEntry("BorderSize", 5);
//     if(size < 0) size = 0;
//     if(size > 20) size = 20;
//     m_borderSize = size;
}

QValueList< AlloyHandler::BorderSize >
AlloyHandler::borderSizes() const
{
    // the list must be sorted
    return QValueList< BorderSize >() << BorderTiny << BorderNormal <<
        BorderLarge << BorderVeryLarge <<  BorderHuge <<
        BorderVeryHuge << BorderOversized;
}

AlloyButton::AlloyButton(AlloyClient *parent, const char *name, const QString& tip, ButtonType type)
    : QButton(parent->widget(), name), m_client(parent), m_lastMouse(0), m_type(type), hover(false), isOnAllDesktops(false), isMaximized(false)
{
    QToolTip::add(this, tip);
    setCursor(arrowCursor);
    setBackgroundMode(NoBackground);
    setFixedSize(BUTTONSIZE, BUTTONSIZE);
//     if (decoFileName) setEmbedImage(decoFileName);
}

AlloyButton::~AlloyButton()
{
//     if (m_deco) delete m_deco;
}

void AlloyButton::setTipText(const QString &tip) {
    QToolTip::remove(this );
    QToolTip::add(this, tip );
}

QSize AlloyButton::sizeHint() const
{
    return QSize(BUTTONSIZE, BUTTONSIZE);
}

void AlloyButton::enterEvent(QEvent *e)
{
    hover = true;
    repaint(false);

    QButton::enterEvent(e);
}

void AlloyButton::leaveEvent(QEvent *e)
{
    hover = false;
    repaint(false);

    QButton::leaveEvent(e);
}

void AlloyButton::mousePressEvent(QMouseEvent* e)
{
    m_lastMouse = e->button();
    // pass on event after changing button to LeftButton
    QMouseEvent me(e->type(), e->pos(), e->globalPos(),
                   LeftButton, e->state());
    QButton::mousePressEvent(&me);
}

void AlloyButton::mouseReleaseEvent(QMouseEvent* e)
{
    m_lastMouse = e->button();
    // pass on event after changing button to LeftButton
    QMouseEvent me(e->type(), e->pos(), e->globalPos(),
                   LeftButton, e->state());
    QButton::mouseReleaseEvent(&me);
}

void AlloyButton::drawButton(QPainter *painter)
{
    if (!AlloyHandler::initialized())
        return;

    bool active = m_client->isActive();

    QPixmap pufferPixmap;
    pufferPixmap.resize(BUTTONSIZE, BUTTONSIZE);
    QPainter pufferPainter(&pufferPixmap);

    if (m_type == ButtonMenu)
    {
        // we paint the mini icon (which is 16 pixels high)
        pufferPainter.drawTiledPixmap(0, -1, 16, 17, active ? *active_titlebar_blank : *inactive_titlebar_blank);
        pufferPainter.drawPixmap((width()-16)/2, (width()-16)/2, m_client->icon().pixmap( QIconSet::Small, QIconSet::Normal ));
    }
    else
    {
        // border
        pufferPainter.drawPixmap(0, 0, active ? *active_button_border : *inactive_button_border);

        // button
        if(m_type == ButtonClose)
        {
            if(hover && !isDown())
            {
                pufferPainter.drawPixmap(1, 1, active ? *active_button_close_hover : *inactive_button_close_hover);
            }
            else if(isDown())
            {
                pufferPainter.drawPixmap(1, 1, active ? *active_button_close_sunken : *inactive_button_close_sunken);
            }
            else
            {
                pufferPainter.drawPixmap(1, 1, active ? *active_button_close_normal : *inactive_button_close_normal);
            }
        }
        else
        {
            if(hover && !isDown())
            {
                pufferPainter.drawPixmap(1, 1, active ? *active_button_common_hover : *inactive_button_common_hover);
            }
            else if(isDown())
            {
                pufferPainter.drawPixmap(1, 1, active ? *active_button_common_sunken : *inactive_button_common_sunken);
            }
            else
            {
                pufferPainter.drawPixmap(1, 1, active ? *active_button_common_normal : *inactive_button_common_normal);
            }
        }

        // deco
        if(isDown())
        {
            if(m_type == ButtonHelp)
            {
                pufferPainter.drawImage((BUTTONSIZE-8)/2, (BUTTONSIZE-8)/2, uic_findImage("question_pressed.png"));
            }
            else if(m_type == ButtonClose)
            {
                pufferPainter.drawImage((BUTTONSIZE-8)/2, (BUTTONSIZE-8)/2, uic_findImage("close_pressed.png"));
            }
            else if(m_type == ButtonMax)
            {
                pufferPainter.drawImage((BUTTONSIZE-8)/2, (BUTTONSIZE-8)/2, uic_findImage(isMaximized ? "restore_pressed.png" : "up_pressed.png"));
            }
            else if(m_type == ButtonMin)
            {
                pufferPainter.drawImage((BUTTONSIZE-8)/2, (BUTTONSIZE-8)/2, uic_findImage("down_pressed.png"));
            }
            else if(m_type == ButtonOnAllDesktops)
            {
                pufferPainter.drawImage((BUTTONSIZE-8)/2, (BUTTONSIZE-8)/2, uic_findImage(isOnAllDesktops ? "unsticky_pressed.png" : "sticky_pressed.png"));
            }
        }
        else
        {
            if(m_type == ButtonHelp)
            {
                pufferPainter.drawImage((BUTTONSIZE-8)/2, (BUTTONSIZE-8)/2, uic_findImage("question.png"));
            }
            else if(m_type == ButtonClose)
            {
                pufferPainter.drawImage((BUTTONSIZE-8)/2, (BUTTONSIZE-8)/2, uic_findImage("close.png"));
            }
            else if(m_type == ButtonMax)
            {
                pufferPainter.drawImage((BUTTONSIZE-8)/2, (BUTTONSIZE-8)/2, uic_findImage(isMaximized ? "restore.png" : "up.png"));
            }
            else if(m_type == ButtonMin)
            {
                pufferPainter.drawImage((BUTTONSIZE-8)/2, (BUTTONSIZE-8)/2, uic_findImage("down.png"));
            }
            else if(m_type == ButtonOnAllDesktops)
            {
                pufferPainter.drawImage((BUTTONSIZE-8)/2, (BUTTONSIZE-8)/2, uic_findImage(isOnAllDesktops ? "unsticky.png" : "sticky.png"));
            }
        }
    }
    pufferPainter.end();

    painter->drawPixmap(0, 0, pufferPixmap);
}

AlloyClient::AlloyClient(KDecorationBridge* bridge, KDecorationFactory* factory)
    : KDecoration(bridge, factory),
      m_titleBar(0)
{
}

void AlloyClient::init()
{
    createMainWidget(WResizeNoErase|WRepaintNoErase);
    widget()->installEventFilter( this );

    // for flicker-free redraws
    widget()->setBackgroundMode(NoBackground);

    //setup layout.
    QGridLayout *mainLayout = new QGridLayout(widget(), 4, 3); // 4x3 grid
    QHBoxLayout *titlelayout = new QHBoxLayout();
    QVBoxLayout *leftButtonLayout = new QVBoxLayout();
    QHBoxLayout *leftButtonLayout2 = new QHBoxLayout();
    leftButtonLayout2->setSpacing(BUTTONSPACING);
    QVBoxLayout *rightButtonLayout = new QVBoxLayout();
    QHBoxLayout *rightButtonLayout2 = new QHBoxLayout();
    rightButtonLayout2->setSpacing(BUTTONSPACING);

    mainLayout->setResizeMode(QLayout::FreeResize);

    m_titleBar = new QSpacerItem(1, TITLESIZE, QSizePolicy::Expanding, QSizePolicy::Fixed);
    m_leftSpacer = new QSpacerItem(AlloyHandler::borderSize(), 1,
                                   QSizePolicy::Expanding, QSizePolicy::Minimum );
    m_rightSpacer = new QSpacerItem(AlloyHandler::borderSize(), 1,
                                    QSizePolicy::Expanding, QSizePolicy::Minimum );
    m_topSpacer = new QSpacerItem( 1, FRAMESIZE_TOP);
    m_bottomSpacer = new QSpacerItem( 1, AlloyHandler::borderSize() );
    m_leftButtonSpacer = new QSpacerItem( 3, TITLESIZE);
    m_rightButtonSpacer = new QSpacerItem( 3, TITLESIZE);
    m_leftButtonSpacer2 = new QSpacerItem(1, (TITLESIZE-BUTTONSIZE)/2,
                                          QSizePolicy::Minimum, QSizePolicy::Fixed);
    m_rightButtonSpacer2 = new QSpacerItem(1, (TITLESIZE-BUTTONSIZE)/2,
                                           QSizePolicy::Minimum, QSizePolicy::Fixed);

    mainLayout->addItem(m_topSpacer, 0, 1);
    mainLayout->addItem(m_bottomSpacer, 3, 1);
    mainLayout->addItem(m_leftSpacer, 1, 0);
    mainLayout->addItem(m_rightSpacer, 1, 2);

    mainLayout->addLayout(titlelayout, 1, 1);
    if( isPreview())
        mainLayout->addWidget( new QLabel( i18n( "<center><b>Alloy</b></center>" ), widget()), 2, 1);
    else
        mainLayout->addItem( new QSpacerItem( 0, 0 ), 2, 1);

    // the window should stretch
    mainLayout->setRowStretch(2, 10);
    mainLayout->setColStretch(1, 10);

    titlelayout->addItem(m_leftButtonSpacer); // Left button margin
    titlelayout->addLayout(leftButtonLayout); // contains the left buttons
    titlelayout->addItem(m_titleBar);
    titlelayout->addLayout(rightButtonLayout); // contains the right buttons
    titlelayout->addItem(m_rightButtonSpacer); // Right button margin

    for (int n=0; n<ButtonTypeCount; n++) m_button[n] = 0;

    leftButtonLayout->addItem(m_leftButtonSpacer2);
    leftButtonLayout->addLayout(leftButtonLayout2);
    addButtons(leftButtonLayout2, options()->customButtonPositions() ? options()->titleButtonsLeft() : QString(default_left) );

    rightButtonLayout->addItem(m_rightButtonSpacer2);
    rightButtonLayout->addLayout(rightButtonLayout2);
    addButtons(rightButtonLayout2, options()->customButtonPositions() ? options()->titleButtonsRight() : QString(default_right) );
}

AlloyClient::~AlloyClient()
{
    for (int n=0; n<ButtonTypeCount; n++) {
        if (m_button[n]) delete m_button[n];
    }
}

bool AlloyClient::eventFilter( QObject* o, QEvent* e )
{
    if ( o != widget() )
        return false;

    switch ( e->type() )
    {
        case QEvent::Resize:
            resizeEvent( static_cast< QResizeEvent* >( e ) );
            return true;
    
        case QEvent::Paint:
            paintEvent( static_cast< QPaintEvent* >( e ) );
            return true;
    
        case QEvent::MouseButtonDblClick:
            mouseDoubleClickEvent( static_cast< QMouseEvent* >( e ) );
            return true;
    
        case QEvent::MouseButtonPress:
            processMousePressEvent( static_cast< QMouseEvent* >( e ) );
            return true;
    
        case QEvent::Show:
            showEvent( static_cast< QShowEvent* >( e ) );
            return true;
    
        default:
                return false;
    }
}

void AlloyClient::addButtons(QBoxLayout *layout, const QString& s)
{
    if (s.length() > 0) {
        for (unsigned n=0; n < s.length(); n++) {
            switch (s[n]) {
              case 'M': // Menu button
                  if (!m_button[ButtonMenu])
                  {
                      m_button[ButtonMenu] = new AlloyButton(this, "menu", i18n("Menu"), ButtonMenu);
                      connect(m_button[ButtonMenu], SIGNAL(pressed()), this, SLOT(menuButtonPressed()));
                      layout->addWidget(m_button[ButtonMenu], 0, Qt::AlignHCenter | Qt::AlignTop);
                  }
                  break;
              case 'S': // OnAllDesktops button
                  if (!m_button[ButtonOnAllDesktops])
                  {
                      m_button[ButtonOnAllDesktops] = new AlloyButton(this, "onAllDesktops", isOnAllDesktops()?i18n("Not On All Desktops"):i18n("On All Desktops"), ButtonOnAllDesktops);
                      m_button[ButtonOnAllDesktops]->setOnAllDesktops(isOnAllDesktops());
                      connect(m_button[ButtonOnAllDesktops], SIGNAL(clicked()), this, SLOT(toggleOnAllDesktops()));
                      layout->addWidget(m_button[ButtonOnAllDesktops], 0, Qt::AlignHCenter | Qt::AlignTop);
                  }
                  break;
              case 'H': // Help button
                  if ((!m_button[ButtonHelp]) && providesContextHelp())
                  {
                      m_button[ButtonHelp] = new AlloyButton(this, "help", i18n("Help"), ButtonHelp);
                      connect(m_button[ButtonHelp], SIGNAL(clicked()), this, SLOT(showContextHelp()));
                      layout->addWidget(m_button[ButtonHelp], 0, Qt::AlignHCenter | Qt::AlignTop);
                  }
                  break;
              case 'I': // Minimize button
                  if ((!m_button[ButtonMin]) && isMinimizable())
                  {
                      m_button[ButtonMin] = new AlloyButton(this, "minimize", i18n("Minimize"), ButtonMin);
                      connect(m_button[ButtonMin], SIGNAL(clicked()), this, SLOT(minimize()));
                      layout->addWidget(m_button[ButtonMin], 0, Qt::AlignHCenter | Qt::AlignTop);
                  }
                  break;
              case 'A': // Maximize button
                  if ((!m_button[ButtonMax]) && isMaximizable())
                  {
                      m_button[ButtonMax] = new AlloyButton(this, "maximize", (maximizeMode()!=MaximizeRestore)?i18n("Minimize"):i18n("Maximize"), ButtonMax);
                      connect(m_button[ButtonMax], SIGNAL(clicked()), this, SLOT(maxButtonPressed()));
                      layout->addWidget(m_button[ButtonMax], 0, Qt::AlignHCenter | Qt::AlignTop);
                  }
                  break;
              case 'X': // Close button
                  if ((!m_button[ButtonClose]) && isCloseable())
                  {
                      m_button[ButtonClose] = new AlloyButton(this, "close", i18n("Close"), ButtonClose);
                      connect(m_button[ButtonClose], SIGNAL(clicked()), this, SLOT(closeWindow()));
                      layout->addWidget(m_button[ButtonClose], 0, Qt::AlignHCenter | Qt::AlignTop);
                  }
                  break;
              case '_': // Spacer item
                  layout->addSpacing(BUTTONSPACING);
            }
	}
    }
}

void AlloyClient::paintEvent(QPaintEvent*)
{
    if (!AlloyHandler::initialized()) return;

    bool active = isActive();

    QPainter painter(widget());

    QColorGroup group;
    group = options()->colorGroup(ColorTitleBar, active);

    //-- draw the titlebar

    QRect titleRect(m_titleBar->geometry());
    QRect topRect(m_topSpacer->geometry());
    QRect leftRect(m_leftSpacer->geometry());
    QRect rightRect(m_rightSpacer->geometry());
    QRect bottomRect(m_bottomSpacer->geometry());

    QColor color2 = group.background();

    //we need caption width...
    QFontMetrics fm(options()->font(active, false));
    int captionMetricWidth = fm.width(caption()); // neccessary width
    int captionWidth = // available, used width
        (titleRect.width()-2*TITLESPACER1 < captionMetricWidth) ? // shrink the caption?
        titleRect.width()-2*TITLESPACER1+1 : captionMetricWidth+1;
        //                              ^^                     ^^ 1px shadow.


    painter.drawTiledPixmap(0, titleRect.top(), titleRect.left()-1, titleRect.bottom()-titleRect.top(),
                            active ? *active_titlebar_blank : *inactive_titlebar_blank);
    painter.drawTiledPixmap(titleRect.right(), titleRect.top(),
                            (width() - 2) - (titleRect.right()) +2, titleRect.bottom()-titleRect.top(),
                            active ? *active_titlebar_blank : *inactive_titlebar_blank);

    if( titleRect.width()-2*TITLESPACER1 > 0) // do we need to paint the caption at all?
    {
        QPixmap pufferPixmap;
        pufferPixmap.resize(captionWidth, titleRect.height()-1 );

        int flags = (pufferPixmap.width() < captionMetricWidth) ? AlignLeft | AlignVCenter : AlignCenter;

        QPainter pufferPainter(&pufferPixmap);
        pufferPainter.drawTiledPixmap(pufferPixmap.rect(),
                                      active ? *active_titlebar_blank : *inactive_titlebar_blank);
        pufferPainter.setFont(options()->font(active, false));

        pufferPainter.translate( 0, 1 ); // move the text 1 pixel down...

        if(AlloyHandler::titleShadow())
        {
            pufferPainter.translate( 1, 1 );
            pufferPainter.setPen( group.background().dark(140) );
            pufferPainter.drawText( pufferPixmap.rect(), flags, caption() );
            pufferPainter.translate( -1, -1 );
        }

        pufferPainter.setPen(options()->color(ColorFont, active));
        pufferPainter.drawText(pufferPixmap.rect(), flags, caption() );
        pufferPainter.end();
        painter.drawPixmap(titleRect.left() + (titleRect.width()-pufferPixmap.width())/2, titleRect.top(),
                           pufferPixmap);
    }

    if( titleRect.width()-2*TITLESPACER1-2*TITLESPACER2-captionMetricWidth > 0)
    {
    painter.drawTiledPixmap(titleRect.left()+TITLESPACER1, titleRect.top(),
                            (titleRect.width()-captionWidth)/2 - TITLESPACER1 - TITLESPACER2,
                                titleRect.bottom()-titleRect.top(),
                            active ? *active_titlebar_grooves : *inactive_titlebar_grooves);
    painter.drawTiledPixmap(titleRect.left() + (titleRect.width()+captionWidth)/2 + TITLESPACER2, titleRect.top(),
                            (titleRect.right()-TITLESPACER1) - (titleRect.left() +
                                (titleRect.width()+captionWidth)/2+TITLESPACER2),
                                titleRect.bottom()-titleRect.top(),
                            active ? *active_titlebar_grooves : *inactive_titlebar_grooves);
    }

    if( titleRect.width()-2*TITLESPACER1-captionMetricWidth > 0)
    {
//     painter.drawTiledPixmap(titleRect.left()+(titleRect.width()-captionWidth)/2 - TITLESPACER2, titleRect.top(),
    painter.drawTiledPixmap(titleRect.left()+(titleRect.width()-captionWidth)/2 - TITLESPACER2, titleRect.top(),
                            TITLESPACER2, titleRect.bottom()-titleRect.top(),
                            active ? *active_titlebar_blank : *inactive_titlebar_blank);
    painter.drawTiledPixmap(titleRect.left() + (titleRect.width()+captionWidth)/2, titleRect.top(),
                            TITLESPACER2, titleRect.bottom()-titleRect.top(),
                            active ? *active_titlebar_blank : *inactive_titlebar_blank);
    }

    if( titleRect.width() > 0)
    {
    painter.drawTiledPixmap(titleRect.left()-1, titleRect.top(), TITLESPACER1+1, titleRect.bottom()-titleRect.top(),
                            active ? *active_titlebar_blank : *inactive_titlebar_blank);
    painter.drawTiledPixmap(titleRect.right() - TITLESPACER1, titleRect.top(), TITLESPACER1, titleRect.bottom()-titleRect.top(),
                            active ? *active_titlebar_blank : *inactive_titlebar_blank);
    }


    // ----draw frame

    // --edges...

    //top/light: hidden if border mode is MaximizedFull
    if(maximizeMode() != MaximizeFull) {
        painter.setPen(hsvRelative(color2, 0, -80, +40) );
        painter.drawLine(0, 0, width()-2, 0);
    }

    // left/top/light
    if(leftRect.width() >= 1)
    {
        painter.drawTiledPixmap(leftRect.left(), titleRect.top(), 1, titleRect.bottom()-titleRect.top(), active ? *active_titlebar_left : *inactive_titlebar_left);
    }

    // left/light + bottom/light + right/light
    painter.setPen(hsvRelative(color2, 0, -50, +20) );
    if(leftRect.width() >= 1)
    {
        painter.drawLine(leftRect.left(), titleRect.bottom(), leftRect.left(), bottomRect.bottom()-1);
    }
    if(bottomRect.height() >= 2)
    {
        painter.drawLine(leftRect.right()+1, bottomRect.top(), rightRect.left()-1, bottomRect.top());
    }
    if(rightRect.width() >= 2)
    {
        painter.drawLine(rightRect.left(), titleRect.bottom()+1, rightRect.left(), bottomRect.top());
    }

    // top/dark + left/dark + bottom/dark + right/dark
    painter.setPen(hsvRelative(color2, 0, 0, -50) );
    painter.drawLine(leftRect.right()+1, titleRect.bottom(),
                     rightRect.left()-1, titleRect.bottom() );
    if(leftRect.width() >= 2)
    {
        painter.drawLine(leftRect.right(), titleRect.bottom(),
                         leftRect.right(), bottomRect.top() );
    }
    if(bottomRect.width() >= 4)
    {
        painter.drawLine(leftRect.left()+1, bottomRect.bottom(),
                         rightRect.right(), bottomRect.bottom() );
    }
    if(rightRect.width() >= 1)
    {
        painter.drawLine(rightRect.right(), titleRect.bottom(),
                         rightRect.right(), bottomRect.bottom());
    }

    // right/top/dark
    if(rightRect.width() >= 1)
    {
        painter.drawTiledPixmap(rightRect.right(), titleRect.top(), 1, titleRect.bottom()-titleRect.top(), active ? *active_titlebar_right : *inactive_titlebar_right);
    }

    // fill the rest in the base color
    painter.setPen(color2);
    if(leftRect.width() >= 2)
    {
        painter.drawPoint(leftRect.right(), bottomRect.top());
    }
    if(rightRect.width() >= 2)
    {
        painter.drawPoint(rightRect.left(), titleRect.bottom());
    }
    if(leftRect.width() >= 1)
    {
        painter.drawPoint(leftRect.left(), bottomRect.bottom());
    }
    if(rightRect.width() >= 1)
    {
        painter.drawPoint(rightRect.right(), topRect.top());
    }

    QRect frame;
    if(leftRect.width() >= 3)
    {
        frame.setCoords(leftRect.left()+1, titleRect.bottom(),
                        leftRect.right()-1, bottomRect.bottom()-1);
        painter.fillRect(frame, color2);
    }
    if(rightRect.width() >= 3)
    {
        frame.setCoords(rightRect.left()+1, titleRect.bottom(),
                        rightRect.right()-1, bottomRect.bottom()-1);
        painter.fillRect(frame, color2);
    }
    if(bottomRect.height() >= 3)
    {
        frame.setCoords(leftRect.right(), bottomRect.top()+1,
                        rightRect.left(), bottomRect.bottom()-1);
        painter.fillRect(frame, color2);
    }

    // resizer details bottom/left/right.
    if(isResizable())
    {
        QPixmap *detailPtr = 0;
        detailPtr =
            // the smaller or the bigger one...?
            (bottomRect.height()<5 || leftRect.width()<5 || rightRect.width()<5) ?
              // active or inactive?
              (active ? active_resizer_detail_2 : inactive_resizer_detail_2)
            : (active ? active_resizer_detail_3 : inactive_resizer_detail_3);
        int detailSize = (bottomRect.height()<5 || leftRect.width()<5 || rightRect.width()<5) ? 2 : 3;

        if(bottomRect.height() >= 3)
        {
            painter.drawPixmap(leftRect.left()+1, bottomRect.bottom()-detailSize,
                               *detailPtr);
            painter.drawPixmap(rightRect.right()-detailSize, bottomRect.bottom()-detailSize,
                               *detailPtr);
            painter.drawPixmap(leftRect.left()+1+detailSize+2, bottomRect.bottom()-detailSize,
                               *detailPtr);
            painter.drawPixmap(rightRect.right()-detailSize-2-detailSize, bottomRect.bottom()-detailSize,
                               *detailPtr);
        }

        if(leftRect.width() >= 3)
        {
            painter.drawPixmap(leftRect.left()+1, bottomRect.bottom()-detailSize-2-detailSize,
                               *detailPtr);
        }
        else if(bottomRect.height() >= 3)
        {
            painter.drawPixmap(leftRect.left()+1+detailSize+2+detailSize+2, bottomRect.bottom()-detailSize,
                               *detailPtr);
        }

        if(rightRect.width() >= 3)
        {
            painter.drawPixmap(rightRect.right()-detailSize, bottomRect.bottom()-detailSize-2-detailSize,
                               *detailPtr);
        }
        else if(bottomRect.height() >= 3)
        {
            painter.drawPixmap(rightRect.right()-detailSize-2-detailSize-2-detailSize,
                               bottomRect.bottom()-detailSize,
                               *detailPtr);
        }
    }
}

void AlloyClient::showEvent(QShowEvent *)
{
    widget()->update();
}

void AlloyClient::resizeEvent(QResizeEvent *)
{
    if ((widget()->isVisibleToTLW()) && (!widget()->testWFlags(WStaticContents))) {
        QRegion region = widget()->rect();
        region.subtract(m_titleBar->geometry());
        widget()->erase(region);
    }
}

void AlloyClient::captionChange()
{
    widget()->repaint(m_titleBar->geometry(), false);
}

void AlloyClient::mouseDoubleClickEvent(QMouseEvent *e)
{
    if (m_titleBar->geometry().contains(e->pos()))
        titlebarDblClickOperation();
}

AlloyClient::Position AlloyClient::mousePosition(const QPoint &point) const
{
    const int corner = 24;
    AlloyClient::Position pos = PositionCenter;

    // for often needed coords..
    QRect titleRect(m_titleBar->geometry());
    QRect topRect(m_topSpacer->geometry());
    QRect leftRect(m_leftSpacer->geometry());
    QRect rightRect(m_rightSpacer->geometry());
    QRect bottomRect(m_bottomSpacer->geometry());

    if((m_leftButtonSpacer2->geometry().height()>0)
           ? (point.y()<=(m_leftButtonSpacer2->geometry().bottom())) : (point.y()<=topRect.bottom()) )
    {
        // inside top frame
        if (point.x() <= corner)                 pos = PositionTopLeft;
        else if (point.x() >= (width()-corner))  pos = PositionTopRight;
        else                                     pos = PositionTop;
    }
    else if(topRect.height()<=0 && point.y()<=2)
    {
        // inside top frame probably at maximized state.
        if ((point.x()>=titleRect.left()) &&
            (point.x()<=titleRect.left()+corner) )    pos = PositionTopLeft;
        else if ((point.x()>=titleRect.right()-corner) &&
                 (point.x()<=titleRect.right()) )     pos = PositionTopRight;
        else if((point.x()>titleRect.left()) &&
                (point.x()<titleRect.right()) )        pos = PositionTop;
    }
    else if(point.y()>=bottomRect.top() )
    {
        // inside handle
        if(point.x() <= corner)                 pos = PositionBottomLeft;
        else if(point.x() >= (width()-corner))  pos = PositionBottomRight;
        else                                     pos = PositionBottom;
    }
    else if(point.x()<=leftRect.right() )
    {
        // on left frame
        if(point.y() <= corner)                 pos = PositionTopLeft;
        else if(point.y() >= (height()-corner)) pos = PositionBottomLeft;
        else                                     pos = PositionLeft;
    }
    else if(point.x()>=rightRect.left() )
    {
        // on right frame
        if(point.y() <= corner)                 pos = PositionTopRight;
        else if(point.y() >= (height()-corner)) pos = PositionBottomRight;
        else                                     pos = PositionRight;
    }
    else
    {
        // inside the frame
        pos = PositionCenter;
    }
    return pos;
}

void AlloyClient::iconChange()
{
    if (m_button[ButtonMenu])
    {
        m_button[ButtonMenu]->repaint(false);
    }
}

void AlloyClient::activeChange()
{
    for (int n=0; n<ButtonTypeCount; n++)
        if (m_button[n]) m_button[n]->repaint(false);
    widget()->repaint(false);
}

void AlloyClient::maximizeChange()
{
    const bool m = (maximizeMode()!=MaximizeRestore);
    if (m_button[ButtonMax])
    {
        m_button[ButtonMax]->setMaximized(m);
        m_button[ButtonMax]->setTipText(m ? i18n("Restore") : i18n("Maximize"));
    }
}

void AlloyClient::desktopChange()
{
    if (m_button[ButtonOnAllDesktops])
    {
        m_button[ButtonOnAllDesktops]->setOnAllDesktops(isOnAllDesktops());
        m_button[ButtonOnAllDesktops]->setTipText(isOnAllDesktops() ? i18n("Not On All Desktops") : i18n("On All Desktops"));
    }
}

void AlloyClient::maxButtonPressed()
{
    if (m_button[ButtonMax])
    {
        switch (m_button[ButtonMax]->lastMousePress())
        {
          case MidButton:
              maximize(maximizeMode() ^ MaximizeVertical );
              break;
          case RightButton:
              maximize(maximizeMode() ^ MaximizeHorizontal );
              break;
          default:
              maximize(maximizeMode() == MaximizeFull ? MaximizeRestore : MaximizeFull );
        }
    }
}

void AlloyClient::menuButtonPressed()
{
    if (m_button[ButtonMenu])
    {
        QPoint pt(m_button[ButtonMenu]->rect().bottomLeft().x(),
                  m_button[ButtonMenu]->rect().bottomLeft().y());
        showWindowMenu(m_button[ButtonMenu]->mapToGlobal(pt));
        m_button[ButtonMenu]->setDown(false);
    }
}

QSize AlloyClient::minimumSize() const
{
    return widget()->minimumSize();
}

void AlloyClient::borders( int& left, int& right, int& top, int& bottom ) const
{
    if( (maximizeMode() == MaximizeFull) && !options()->moveResizeMaximizedWindows() )
    {
            m_leftSpacer->changeSize( 0, 1, QSizePolicy::Expanding, QSizePolicy::Minimum );
            m_rightSpacer->changeSize( 0, 1, QSizePolicy::Expanding, QSizePolicy::Minimum );
            m_leftButtonSpacer->changeSize( 0, 1);
            m_rightButtonSpacer->changeSize( 0, 1);
            m_topSpacer->changeSize(1, 0);
            m_bottomSpacer->changeSize(1, 3);
            m_titleBar->changeSize(1, TITLESIZE-(TITLESIZE-BUTTONSIZE)/2, QSizePolicy::Expanding, QSizePolicy::Fixed);
            m_leftButtonSpacer2->changeSize( 1, 0,
                                            QSizePolicy::Minimum, QSizePolicy::Fixed);
            m_rightButtonSpacer2->changeSize( 1, 0,
                                            QSizePolicy::Minimum, QSizePolicy::Fixed);

            left = right = 0;
            bottom = 3;
            top = TITLESIZE-(TITLESIZE-BUTTONSIZE)/2;
    } else if ( (maximizeMode() == MaximizeHorizontal) && !options()->moveResizeMaximizedWindows() ) {
            // We've been maximized - shrink the sideways spacers
            m_leftSpacer->changeSize( 1, 1, QSizePolicy::Expanding, QSizePolicy::Minimum );
            m_rightSpacer->changeSize( 1, 1, QSizePolicy::Expanding, QSizePolicy::Minimum );
            m_topSpacer->changeSize(1, FRAMESIZE_TOP);
            m_bottomSpacer->changeSize(1, AlloyHandler::borderSize() );
            m_leftButtonSpacer->changeSize( 0, 1);
            m_rightButtonSpacer->changeSize( 0, 1);
            m_titleBar->changeSize(1, TITLESIZE, QSizePolicy::Expanding, QSizePolicy::Fixed);
            m_leftButtonSpacer2->changeSize(1, (TITLESIZE-BUTTONSIZE)/2,
                                            QSizePolicy::Minimum, QSizePolicy::Fixed);
            m_rightButtonSpacer2->changeSize(1, (TITLESIZE-BUTTONSIZE)/2,
                                             QSizePolicy::Minimum, QSizePolicy::Fixed);

            left = right = 1;
            bottom = AlloyHandler::borderSize();
            top = FRAMESIZE_TOP + TITLESIZE;
    } else if ( (maximizeMode() == MaximizeVertical) && !options()->moveResizeMaximizedWindows() ) {
            // We've been maximized - shrink the sideways spacers
            m_leftSpacer->changeSize(AlloyHandler::borderSize(), 1,
                                     QSizePolicy::Expanding, QSizePolicy::Minimum );
            m_rightSpacer->changeSize(AlloyHandler::borderSize(), 1,
                                      QSizePolicy::Expanding, QSizePolicy::Minimum );
            m_topSpacer->changeSize(1, FRAMESIZE_TOP);
            m_bottomSpacer->changeSize(1, AlloyHandler::borderSize());
            m_leftButtonSpacer->changeSize( 3, 1);
            m_rightButtonSpacer->changeSize( 3, 1);
            m_titleBar->changeSize(1, TITLESIZE, QSizePolicy::Expanding, QSizePolicy::Fixed);
            m_leftButtonSpacer2->changeSize(1, (TITLESIZE-BUTTONSIZE)/2,
                                            QSizePolicy::Minimum, QSizePolicy::Fixed);
            m_rightButtonSpacer2->changeSize(1, (TITLESIZE-BUTTONSIZE)/2,
                                             QSizePolicy::Minimum, QSizePolicy::Fixed);

            left = right = AlloyHandler::borderSize();
            bottom = AlloyHandler::borderSize();
            top = FRAMESIZE_TOP + TITLESIZE;
    } else {
            // We've been restored - enlarge the sideways spacers
            m_leftSpacer->changeSize(AlloyHandler::borderSize(), 1,
                                     QSizePolicy::Expanding, QSizePolicy::Minimum );
            m_rightSpacer->changeSize(AlloyHandler::borderSize(), 1,
                                      QSizePolicy::Expanding, QSizePolicy::Minimum );
            m_topSpacer->changeSize(1, FRAMESIZE_TOP);
            m_bottomSpacer->changeSize(1, AlloyHandler::borderSize());
            m_leftButtonSpacer->changeSize( 3, 1);
            m_rightButtonSpacer->changeSize( 3, 1);
            m_titleBar->changeSize(1, TITLESIZE, QSizePolicy::Expanding, QSizePolicy::Fixed);
            m_leftButtonSpacer2->changeSize(1, (TITLESIZE-BUTTONSIZE)/2,
                                            QSizePolicy::Minimum, QSizePolicy::Fixed);
            m_rightButtonSpacer2->changeSize(1, (TITLESIZE-BUTTONSIZE)/2,
                                             QSizePolicy::Minimum, QSizePolicy::Fixed);

            left = right = bottom = AlloyHandler::borderSize();
            top = FRAMESIZE_TOP + TITLESIZE;
    }

    // activate updated layout
    widget()->layout()->activate();
}

void AlloyClient::reset( unsigned long changed )
{
//     qDebug("implement AlloyClient::reset() !");
}

void AlloyClient::resize( const QSize& s )
{
    widget()->resize( s );
}

//////////////////////////////////////////////////////////////////////////////
// Plugin Stuff                                                             //
//////////////////////////////////////////////////////////////////////////////

extern "C"
{
    KDecorationFactory *create_factory()
    {
        return new AlloyHandler();
    }
}

#include "alloyclient.moc"
