/***************************************************************************
*   Copyright (C) 2003 by Sandro Giessl                                   *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/

#ifndef ALLOYCLIENT_H
#define ALLOYCLIENT_H

#include <kdecoration.h>
#include <kdecorationfactory.h>
#include <qbutton.h>

class QGridLayout;
class QSpacerItem;
class QPoint;
class AlloyClient;

enum ButtonType {
    ButtonHelp=0,
    ButtonMax,
    ButtonMin,
    ButtonClose,
    ButtonMenu,
    ButtonOnAllDesktops,
    ButtonTypeCount
};

enum BorderMode {
    MaximizedHorizontal=0,
    MaximizedVertical,
    MaximizedFull,
    Normal
};

class AlloyHandler: public QObject, public KDecorationFactory
{
public:
    AlloyHandler();
    ~AlloyHandler();
    virtual bool reset( unsigned long changed ); ///< Reset the handler.

    virtual KDecoration* createDecoration( KDecorationBridge* );

    static bool initialized() { return m_initialized; };

    static bool titleShadow() { return m_titleShadow; };
    static int  borderSize() { return m_borderSize; };

    QValueList< AlloyHandler::BorderSize >  borderSizes() const;
private:
    void readConfig(); ///< Read in the configuration file.

    static bool m_titleShadow;
    static int  m_borderSize;

    static bool m_initialized;
};

class AlloyButton : public QButton
{
public:
    AlloyButton(AlloyClient *parent=0, const char *name=0, const QString &tip=NULL, ButtonType type = ButtonHelp);
    ~AlloyButton();

    void setTipText(const QString &tip);

    void setOnAllDesktops(bool oad) { isOnAllDesktops = oad; repaint(false); }
    void setMaximized(bool maximized) { isMaximized = maximized; repaint(false); }
    QSize sizeHint() const; ///< Return size hint.
    int lastMousePress() const { return m_lastMouse; };
    void reset() { repaint(false); };
    AlloyClient* client() { return m_client; }

private:
    void enterEvent(QEvent *e); ///< Mouse has entered the button.
    void leaveEvent(QEvent *e); ///< Mouse has left the button.
    void mousePressEvent(QMouseEvent *e); ///< Button has been pressed.
    void mouseReleaseEvent(QMouseEvent *e); ///< Button has been released.
    void drawButton(QPainter *painter); ///< Draw the button.

private:
    AlloyClient *m_client;
    int m_lastMouse;

    ButtonType m_type;
    bool hover;
    bool isOnAllDesktops, isMaximized;
};


class AlloyClient : public KDecoration
{
    Q_OBJECT
public:
    AlloyClient(KDecorationBridge* bridge, KDecorationFactory* factory);
    virtual void init();
    ~AlloyClient();

    virtual bool eventFilter( QObject* o, QEvent* e );

    virtual QSize minimumSize() const;
    virtual void borders( int& left, int& right, int& top, int& bottom ) const;
protected:
    virtual void desktopChange(); ///< OnAllDesktops state has changed.
    virtual void activeChange(); ///< window active state has changed.
    virtual void captionChange(); ///< The title has changed.
    virtual void maximizeChange(); ///< Maximized state has changed.
    virtual void iconChange(); ///< The title has changed.
    virtual void shadeChange() {};

    virtual void paintEvent(QPaintEvent *e); ///< Repaint the window.
    virtual void showEvent(QShowEvent *);

    virtual void resize(const QSize&);

    virtual void reset( unsigned long changed );
private:
    void addButtons(QBoxLayout* layout, const QString& buttons); ///< Add buttons to title layout.
    void resizeEvent(QResizeEvent *e); ///< Window is being resized.
    void mouseDoubleClickEvent(QMouseEvent *e); ///< Doubleclick on title.
    Position mousePosition(const QPoint &point) const; ///< Return logical mouse position.

private slots:
    void maxButtonPressed(); ///< Max button was pressed.
    void menuButtonPressed(); ///< Menu button was pressed (popup the menu).

private:
    AlloyButton *m_button[ButtonTypeCount];
    QSpacerItem *m_titleBar, *m_leftSpacer, *m_rightSpacer, *m_topSpacer, *m_bottomSpacer, *m_leftButtonSpacer, *m_leftButtonSpacer2, *m_rightButtonSpacer, *m_rightButtonSpacer2;
};

#endif // ALLOYCLIENT_H
