/***************************************************************************
*   Copyright (C) 2003 by Sandro Giessl                                   *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/

#include <kpixmap.h>
#include <kpixmapeffect.h>

#include <qcolor.h>
#include <qpainter.h>

#include "misc.h"

QColor hsvRelative(QColor& baseColor, int relativeH, int relativeS, int relativeV)
{
    int h, s, v;
    baseColor.hsv(&h, &s, &v);

    h = h + relativeH;
    s = s + relativeS;
    v = v + relativeV;

    if(h < 0) { h = 0; }
    else if(h > 359) { h = 359; }
    if(s < 0) { s = 0; }
    else if(s > 255) { s = 255; }
    if(v < 0) { v = 0; }
    else if(v > 255) { v = 255; }

    QColor c;
    c.setHsv( h, s, v );
    return c;
}

void drawButtonBorder(QPainter* painter, QColor& color) // ~ OK
{

    KPixmap tempPixmap;

    tempPixmap.resize(1, 14);
    KPixmapEffect::gradient(tempPixmap,
                            hsvRelative(color, 0, -46, -30),
                            hsvRelative(color, 0, -10, +1),
                            KPixmapEffect::VerticalGradient);
    painter->drawPixmap(0, 1, tempPixmap);
    KPixmapEffect::gradient(tempPixmap,
                            hsvRelative(color, 0, -48, +29),
                            hsvRelative(color, 0, -97, +34),
                            KPixmapEffect::VerticalGradient);
    painter->drawPixmap(15, 1, tempPixmap);

    tempPixmap.resize(14, 1);
    KPixmapEffect::gradient(tempPixmap,
                            hsvRelative(color, 0, -48, -25),
                            hsvRelative(color, 0, -47, +26),
                            KPixmapEffect::HorizontalGradient);
    painter->drawPixmap(1, 0, tempPixmap);
    KPixmapEffect::gradient(tempPixmap,
                            hsvRelative(color, 0, -9, +5),
                            hsvRelative(color, 0, -92, +31),
                            KPixmapEffect::HorizontalGradient);
    painter->drawPixmap(1, 15, tempPixmap);


    painter->setPen(hsvRelative(color, 0, -48, +12) );
    painter->drawPoint(0,0);
    painter->setPen(hsvRelative(color, 0, -48, +31) );
    painter->drawPoint(15,0);
    painter->setPen(hsvRelative(color, 0, -7, +4) );
    painter->drawPoint(0,15);
    painter->setPen(hsvRelative(color, 0, -36, +13) );
    painter->drawPoint(15,15);
}

void drawButtonBase(QPainter* painter, QColor& color, QColor& blendColor, int state)
{
    if(state < 1 && state > 3)
        return;

    int i;
    KPixmap tempPixmap;

    QColor buttonColor = color;

    // normal button...
    QRgb rgb = QColor(color).rgb();
    QRgb rgb_b = hsvRelative(blendColor, 0, 0, +90).rgb();
    int alpha = 170;
    int inv_alpha = 255 - alpha;

    switch(state)
    {
        case 1: // normal: blend color with blendColor then jump to case 2 - no break;!
            buttonColor = QColor( qRgb(qRed(rgb_b)*inv_alpha/255 + qRed(rgb)*alpha/255,
                          qGreen(rgb_b)*inv_alpha/255 + qGreen(rgb)*alpha/255,
                          qBlue(rgb_b)*inv_alpha/255 + qBlue(rgb)*alpha/255) );

        case 2: // hover, acts also for normal buttons(different color...)
            tempPixmap.resize(1, 12);
            KPixmapEffect::gradient(tempPixmap,
                                    hsvRelative(buttonColor, 0, -130, +24),
                                    hsvRelative(buttonColor, 0, -35, -68),
                                    KPixmapEffect::VerticalGradient);
            painter->drawPixmap(0, 1, tempPixmap);
            KPixmapEffect::gradient(tempPixmap,
                                    hsvRelative(buttonColor, 0, -74, -24),
                                    hsvRelative(buttonColor, 0, -23, -120),
                                    KPixmapEffect::VerticalGradient);
            painter->drawPixmap(13, 1, tempPixmap);

            tempPixmap.resize(12, 1);
            KPixmapEffect::gradient(tempPixmap,
                                    hsvRelative(buttonColor, 0, -130, +22),
                                    hsvRelative(buttonColor, 0, -82, -14),
                                    KPixmapEffect::HorizontalGradient);
            painter->drawPixmap(1, 0, tempPixmap);
            KPixmapEffect::gradient(tempPixmap,
                                    hsvRelative(buttonColor, 0, -24, -85),
                                    hsvRelative(buttonColor, 0, -21, -126),
                                    KPixmapEffect::HorizontalGradient);
            painter->drawPixmap(1, 13, tempPixmap);

            for(i=0;i<12;i++)
            {
                tempPixmap.resize(1, 12);
                KPixmapEffect::gradient(tempPixmap,
                                        hsvRelative(buttonColor, 0, -86+i, -9-i),
                                        hsvRelative(buttonColor, 0, -31, -74-i),
                                        KPixmapEffect::VerticalGradient);
                painter->drawPixmap(1+i, 1, tempPixmap);
            }

            painter->setPen(hsvRelative(buttonColor, 0, -108, -3) );
            painter->drawPoint(0,0);
            painter->setPen(hsvRelative(buttonColor, 0, -80, -16) );
            painter->drawPoint(13,0);
            painter->setPen(hsvRelative(buttonColor, 0, -26, -80) );
            painter->drawPoint(0,13);
            painter->setPen(hsvRelative(buttonColor, 0, -47, -107) );
            painter->drawPoint(13,13);
        break;

        case 3: // pressed
            tempPixmap.resize(1, 12);
            KPixmapEffect::gradient(tempPixmap,
                                    hsvRelative(buttonColor, 0, -43, -83),
                                    hsvRelative(buttonColor, 0, -12, -87),
                                    KPixmapEffect::VerticalGradient);
            painter->drawPixmap(0, 1, tempPixmap);
            KPixmapEffect::gradient(tempPixmap,
                                    hsvRelative(buttonColor, 0, -46, -48),
                                    hsvRelative(buttonColor, 0, -118, -18),
                                    KPixmapEffect::VerticalGradient);
            painter->drawPixmap(13, 1, tempPixmap);

            tempPixmap.resize(12, 1);
            KPixmapEffect::gradient(tempPixmap,
                                    hsvRelative(buttonColor, 0, -46, -80),
                                    hsvRelative(buttonColor, 0, -45, -48),
                                    KPixmapEffect::HorizontalGradient);
            painter->drawPixmap(1, 0, tempPixmap);
            KPixmapEffect::gradient(tempPixmap,
                                    hsvRelative(buttonColor, 0, -15, -88),
                                    hsvRelative(buttonColor, 0, -118, -24),
                                    KPixmapEffect::HorizontalGradient);
            painter->drawPixmap(1, 13, tempPixmap);

            for(i=0;i<12;i++)
            {
                tempPixmap.resize(1, 12);
                KPixmapEffect::gradient(tempPixmap,
                                        hsvRelative(buttonColor, 0, -43, -46-i/6),
                                        hsvRelative(buttonColor, 0, -13-i, -85-i/6),
                                        KPixmapEffect::VerticalGradient);
                painter->drawPixmap(1+i, 1, tempPixmap);
            }

            painter->setPen(hsvRelative(buttonColor, 0, -44, -65) );
            painter->drawPoint(0,0);
            painter->setPen(hsvRelative(buttonColor, 0, -46, -46) );
            painter->drawPoint(13,0);
            painter->setPen(hsvRelative(buttonColor, 0, -8, -88) );
            painter->drawPoint(0,13);
            painter->setPen(hsvRelative(buttonColor, 0, -82, -66) );
            painter->drawPoint(13,13);
        break;
    }
}

