/***************************************************************************
 * alloystyle.cpp **********************************************************
 *                                                                         *
 * Copyright (C) 2003 by Simone Rota <simonerota@operamail.com>            *
 * Copyright (C) 2002, 2003 David Johnson <david@usermode.org>             *
 * Copyright (C) 2003 Daniel Brownlees <danielb@orcon.net.nz>              *
 *                                                                         *
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kdrawutil.h>
#include <kpixmap.h>
#include <kpixmapeffect.h>

#include <qapplication.h>
#include <qintdict.h>
#include <qpainter.h>
#include <qpointarray.h>
#include <qsettings.h>

#include <qcombobox.h>
#include <qheader.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qprogressbar.h>
#include <qpushbutton.h>
#include <qscrollbar.h>
#include <qslider.h>
#include <qtabbar.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qsplitter.h>

#include <qstyleplugin.h>

#include "alloystyle.h"

// some convenient constants
static const int ITEMFRAME       = 1;
static const int ITEMHMARGIN     = 3;
static const int ITEMVMARGIN     = 2;
static const int ARROWMARGIN     = 6;
static const int RIGHTBORDER     = 6;
static const int MINICONSIZE     = 16;
static const int CHECKSIZE       = 7;
static const int MAXGRADIENTSIZE = 64;

static const char* TOOLBARWIDGET = "kde toolbar widget";
static const char* KFILEDIALOGTOOLBAR = "KFileDialog::toolbar";

static QBitmap radiooffshadow, radiooffdark, radiooffmid;
static QBitmap radioonshadow, radioondark, radioonmid;
static QBitmap radiomask;

static unsigned char radiooff_shadow_bits[] = {
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00};

static unsigned char radiooff_dark_bits[] = {
    0xf0, 0x01, 0x0c, 0x06, 0x02, 0x08, 0x02, 0x08, 0x01, 0x10, 0x01, 0x10,
    0x01, 0x10, 0x01, 0x10, 0x01, 0x10, 0x02, 0x08, 0x02, 0x08, 0x0c, 0x06,
    0xf0, 0x01};

static unsigned char radiooff_mid_bits[] = {
    0x08, 0x02, 0x10, 0x01, 0x04, 0x04, 0x01, 0x10, 0x02, 0x08, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x02, 0x08, 0x01, 0x10, 0x04, 0x04, 0x10, 0x01,
    0x08, 0x02};

static unsigned char radioon_shadow_bits[] = {
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x00, 0xe0, 0x00,
   0xf0, 0x01, 0xe0, 0x00, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00};

static unsigned char radioon_dark_bits[] = {
   0xf0, 0x01, 0x0c, 0x06, 0x02, 0x08, 0x02, 0x08, 0xb1, 0x11, 0x11, 0x11,
   0x01, 0x10, 0x11, 0x11, 0xb1, 0x11, 0x02, 0x08, 0x02, 0x08, 0x0c, 0x06,
   0xf0, 0x01};

static unsigned char radioon_mid_bits[] = {
   0x08, 0x02, 0x10, 0x01, 0x04, 0x04, 0xe1, 0x10, 0x02, 0x08, 0x08, 0x02,
   0x08, 0x02, 0x08, 0x02, 0x02, 0x08, 0xe1, 0x10, 0x04, 0x04, 0x10, 0x01,
   0x08, 0x02};

static unsigned char radiomask_bits[] = {
    0xf8, 0x03, 0xfc, 0x07, 0xfe, 0x0f, 0xff, 0x1f, 0xff, 0x1f, 0xff, 0x1f,
    0xff, 0x1f, 0xff, 0x1f, 0xff, 0x1f, 0xff, 0x1f, 0xfe, 0x0f, 0xfc, 0x07,
    0xf8, 0x03};

QMap<unsigned int, QIntDict<GradientSet> > darkgradients; // darkgradients[color][size]
QMap<unsigned int, QIntDict<GradientSet> > lightgradients; // lightgradients[color][size]

//////////////////////////////////////////////////////////////////////////////
// Construction, Destruction, Initialization                                //
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// Alloy()
// -----------
// Constructor

Alloy::Alloy(bool highcolor)
    : KStyle(FilledFrameWorkaround | AllowMenuTransparency ,ThreeButtonScrollBar),
            highlightwidget(0), highcolor(highcolor && (QPixmap::defaultDepth() > 8)),
            lightgradientvalue(0), darkgradientvalue(0)
{
    radiooffshadow = QBitmap(13, 13, radiooff_shadow_bits, true);
    radiooffdark   = QBitmap(13, 13, radiooff_dark_bits, true);
    radiooffmid    = QBitmap(13, 13, radiooff_mid_bits, true);
    radioonshadow  = QBitmap(13, 13, radioon_shadow_bits, true);
    radioondark    = QBitmap(13, 13, radioon_dark_bits, true);
    radioonmid     = QBitmap(13, 13, radioon_mid_bits, true);
    radiomask      = QBitmap(13, 13, radiomask_bits, true);
    radiomask.setMask(radiomask);
}

Alloy::~Alloy() { ; }

//////////////////////////////////////////////////////////////////////////////
// Polishing                                                                //
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// polish()
// --------
// Initialize the appearance of a widget

void Alloy::polish(QWidget *widget)
{
    if (widget->inherits("QPushButton")) {
        widget->installEventFilter(this);
    } else if (widget->inherits("QToolBarExtensionWidget")) {
        widget->setBackgroundMode(NoBackground);
        widget->installEventFilter(this);
    } else if (!qstrcmp(widget->name(), TOOLBARWIDGET)) {
        // custom widgets in the toolbar
        widget->setBackgroundMode(NoBackground);
        widget->installEventFilter(this);
    } else if (!qstrcmp(widget->name(), KFILEDIALOGTOOLBAR)) {
        widget->setBackgroundMode(NoBackground);
        widget->installEventFilter(this);
    } else if (widget->inherits("KToolBarSeparator")) {
        widget->setBackgroundMode(NoBackground);
        widget->installEventFilter(this);
    } else if (widget->inherits("QMenuBar") ||
               widget->inherits("KToolBarButton")) {
        widget->setBackgroundMode(NoBackground);
    }
    KStyle::polish(widget);
}

//////////////////////////////////////////////////////////////////////////////
// polish()
// --------
// Initialize the palette

void Alloy::polish(QPalette &pal)
{
    // clear out gradients on a color change
    darkgradients.clear();
    lightgradients.clear();

    // (re)load any any custom colours / style settings
    QSettings settings;

    darkgradientvalue = settings.readNumEntry("/alloystyle/Settings/darkGradientValue", 10);
    lightgradientvalue = settings.readNumEntry("/alloystyle/Settings/lightGradientValue", 3);

    //
    // HACK! - Qt's QColorGroup could really do with some more colours to play with... :)
    //   Seeing as we are limited with available colours and Alloy looks better (IMO)
    //   with more colours, we allow the user to override colours for some widgets in
    //   alloystylerc if they wish (note this has side effects).
    //

    // scrollbar slider, progressbars
    primarycolor.setNamedColor(settings.readEntry("/alloystyle/Settings/primaryColor", QString::null));
    // mouse rollover highlighting of widgets
    rollovercolor.setNamedColor(settings.readEntry("/alloystyle/Settings/rolloverColor", QString::null));
    // slider handle, checkbox tick, selected radiobutton
    selectioncolor.setNamedColor(settings.readEntry("/alloystyle/Settings/selectionColor", QString::null));

    //
    //  Java Alloy also uses these:
    //      secondarycolor   // background, buttons - we use QColorGroup::button() or background()
    //      hightlightcolor  // selected text, widget focus rect - we use QColorGroup::highlight()
    //      desktopcolor     // this is the 'desktop' (mainwindow background) in an MDI app (unneeded)
    //

    QStyle::polish(pal);
}

//////////////////////////////////////////////////////////////////////////////
// unPolish()
// ----------
// Undo the initialization of a widget's appearance

void Alloy::unPolish(QWidget *widget)
{
    if (widget->inherits("QPushButton")) {
        widget->removeEventFilter(this);
    } else if (widget->inherits("QToolBarExtensionWidget")) {
        widget->setBackgroundMode(PaletteBackground);
        widget->removeEventFilter(this);
    } else if (!qstrcmp(widget->name(), TOOLBARWIDGET)) {
        widget->setBackgroundMode(PaletteBackground);
        widget->removeEventFilter(this);
    } else if (!qstrcmp(widget->name(), KFILEDIALOGTOOLBAR)) {
        widget->setBackgroundMode(PaletteBackground);
        widget->removeEventFilter(this);
    } else if (widget->inherits("KToolBarSeparator")) {
        widget->setBackgroundMode(PaletteBackground);
        widget->removeEventFilter(this);
    } else if (widget->inherits("QMenuBar") ||
               widget->inherits("KToolBarButton")) {
        widget->setBackgroundMode(QWidget::PaletteBackground);
    }
    KStyle::unPolish(widget);
}

//////////////////////////////////////////////////////////////////////////////
// Drawing                                                                  //
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// drawAlloyBevel()
// ----------------
// Draw the basic Alloy bevel

void Alloy::drawAlloyBevel(QPainter *painter,
                              int x, int y, int w, int h,
                              const QColorGroup &group,
                              bool sunken,
                              bool horizontal,
                              bool reverse) const
{
    int x2 = x + w - 1;
    int y2 = y + h - 1;
    QPen oldpen = painter->pen();

    painter->setPen(group.dark());
    painter->drawRect(x, y, w, h);

    painter->setPen(sunken ? group.mid() : group.light());
    painter->drawLine(x+1, y+1, x2-1, y+1);
    painter->drawLine(x+1, y+2, x+1, y2-1);

    painter->setPen(sunken ? group.light() : group.mid());
    painter->drawLine(x+1, y2-1, x2-1, y2-1);
    painter->drawLine(x2-1, y+1, x2-1, y2-2);

    if (!sunken) {
        drawAlloyGradient(painter, QRect(x+2, y+2, w-4, h-4),
                         group.button(), horizontal,
                         0, 0, w-2, h-2, reverse, true);
    } else {
        // sunken bevels don't get gradients
        painter->fillRect(x+2, y+2, w-4, h-4, sunken ?
                          group.midlight() : group.button());
    }
    painter->setPen(oldpen);
}

//////////////////////////////////////////////////////////////////////////////
// drawAlloyButton()
// ----------------
// Draw the basic Alloy button. Also used for tabs

void Alloy::drawAlloyButton(QPainter *painter,
                               int x, int y, int w, int h,
                               const QColorGroup &group,
                               bool sunken, bool istab, bool filled,
                               bool highlight) const
{

    // if no extended colors are set, default from the current ColorGroup
    QColor rcolor(rollovercolor.isValid() ? rollovercolor : group.highlight());

    int x2 = x + w - 1;
    int y2 = y + h - 1;

    painter->setPen(group.mid());
    painter->drawLine(x, y,  x2-1, y);
    painter->drawLine(x, y+1, x, y2-1);

    painter->setPen(group.dark());
    painter->drawRect(x+1, y+1, w-2, h-2);

    painter->setPen(sunken ? group.mid() : group.light());
    painter->drawLine(x+2, y+2, x2-3, y+2);
    painter->drawLine(x+2, y+3, x+2, y2-3);

    painter->setPen(sunken ? group.light() : group.mid());
    painter->drawLine(x2-2, y+2, x2-2, y2-3);


    painter->setPen(group.background());
    painter->drawPoint(x, y);
    painter->drawPoint(x2, y);
    painter->drawPoint(x2-1, y);
    painter->drawPoint(x2, y+1);

    painter->setPen(group.mid().dark(85));
    painter->drawPoint(x+1, y);
    painter->drawPoint(x+1, y+1);
    painter->drawPoint(x, y+1);
    painter->drawPoint(x2-2, y);
    painter->drawPoint(x2-1, y+1);

    painter->setPen(group.mid().dark(87));
    painter->drawPoint(x+3, y+2);
    painter->drawPoint(x+2, y+3);
    painter->drawPoint(x2-3, y+2);

    painter->setPen(group.mid().dark(90));
    painter->drawPoint(x+2, y);
    painter->drawPoint(x, y+2);
    painter->drawPoint(x2-3, y);

    painter->setPen(group.mid().dark(120));
    painter->drawPoint(x+2, y+1);
    painter->drawPoint(x+2, y+2);
    painter->drawPoint(x+1, y+2);
    painter->drawPoint(x2-2, y+1);
    painter->drawPoint(x2-1, y+2);

    painter->setPen(group.mid().dark(140));
    painter->drawPoint(x+3, y+1);
    painter->drawPoint(x+1, y+3);
    painter->drawPoint(x2-3, y+1);
    painter->drawPoint(x2-2, y+2);
    painter->drawPoint(x2-1, y+3);

    if (!istab){

            painter->setPen(group.light());
            painter->drawLine(x, y2, x2, y2);
            painter->drawLine(x2, y+2, x2, y2-1);

            painter->setPen(group.background());
            painter->drawPoint(x, y2-2);
            painter->drawPoint(x, y2-1);
            painter->drawPoint(x, y2);
            painter->drawPoint(x+1, y2);
            painter->drawPoint(x+2, y2);
            painter->drawPoint(x2, y2);
            painter->drawPoint(x2-1, y2);
            painter->drawPoint(x2, y2-1);
            painter->drawPoint(x2-1, y2-1);

            painter->setPen(group.mid().dark(85));
            painter->drawPoint(x, y2-2);
            painter->drawPoint(x+1, y2-1);

            painter->setPen(group.mid().dark(90));
            painter->drawPoint(x2-1, y2-2);
            painter->drawPoint(x2-2, y2-1);
            painter->drawPoint(x, y2-3);
            painter->drawPoint(x+3, y2-2);
            painter->drawPoint(x2-3, y2-2);

            painter->setPen(group.mid().dark(120));
            painter->drawPoint(x2-1, y+2);
            painter->drawPoint(x+1, y2-2);
            painter->drawPoint(x+2, y2-2);
            painter->drawPoint(x+2, y2-1);
            painter->drawPoint(x2-1, y2-2);
            painter->drawPoint(x2-2, y2-1);

            painter->setPen(group.mid().dark(140));
            painter->drawPoint(x+1, y2-3);
            painter->drawPoint(x+3, y2-1);
            painter->drawPoint(x2-1, y2-3);
            painter->drawPoint(x2-2, y2-2);
            painter->drawPoint(x2-3, y2-1);

    } else {
            painter->setPen(group.dark());
            painter->drawPoint(x, y2-1);
            painter->drawPoint(x-1, y2-1);
            painter->drawPoint(x2, y2-1);
            painter->drawPoint(x2+1, y2-1);

            painter->setPen(group.mid());
            painter->drawPoint(x2-2, y2-2);
            painter->drawPoint(x2-2, y2-1);

    }

    if (filled && istab) {
            drawAlloyGradient(painter, QRect(x+3, y+3, w-6, h-5),
                              group.button().dark(106),
                              false, 0, 0, w-6, h-5, sunken);
    }
    else if (filled) {
            drawAlloyGradient(painter, QRect(x+3, y+3, w-6, h-5),
                              group.button(),
                              false, 0, 0, w-6, h-5, sunken);
    }
    else {
            painter->fillRect(x+3, y+3, w-6, h-6, sunken ?
                          group.midlight() : group.button());
    }


    if (highlight) {
        drawAlloyRect(painter, x+2, y+2, w-4, h-4, rcolor);
        painter->setPen(rcolor.dark(150));
        painter->drawPoint(x+2, y+2);
        painter->drawPoint(x+2, y2-2);
        painter->drawPoint(x2-2, y+2);
        painter->drawPoint(x2-2, y2-2);
    }

}

//////////////////////////////////////////////////////////////////////////////
// drawAlloyPanel()
// ----------------
// Draw the basic Alloy panel

void Alloy::drawAlloyPanel(QPainter *painter,
                              int x, int y, int w, int h,
                              const QColorGroup &group,
                              bool sunken,
                              const QBrush *fill) const
{
    int x2 = x + w - 1;
    int y2 = y + h - 1;

    painter->save();
    if (sunken) {
        painter->setPen(group.dark());
        painter->drawRect(x+1, y+1, w-2, h-2);
//        painter->drawLine(x+1, y2-1, x2-1, y2-1);
//        painter->drawLine(x2-1, y+1, x2-1, y2-2);
        painter->setPen(group.mid());
        painter->drawLine(x, y, x2-1, y);
        painter->drawLine(x, y+1, x, y2-1);
        painter->setPen(group.light());
        painter->drawLine(x, y2, x2, y2);
        painter->drawLine(x2, y, x2, y2-1);
    } else {
        painter->setPen(group.dark());
        painter->drawRect(x, y, w, h);
        painter->setPen(group.mid());
        painter->drawLine(x+1, y2-1, x2-1, y2-1);
        painter->drawLine(x2-1, y+1, x2-1, y2-2);
        painter->setPen(group.light());
        painter->drawLine(x+1, y+1, x+1, y2-2);
        painter->drawLine(x+2, y+1, x2-2, y+1);
    }
    if (fill) {
        painter->fillRect(QRect(x+2, y+2, w-4, h-4), *fill);
    }
    painter->restore();
}

//////////////////////////////////////////////////////////////////////////////
// drawPrimitive()
// ---------------
// Draw the element

void Alloy::drawPrimitive(PrimitiveElement element,
                              QPainter *painter,
                              const QRect &rect,
                              const QColorGroup &group,
                              SFlags flags,
                              const QStyleOption &option) const
{
    bool down    = flags & Style_Down;
    bool on      = flags & Style_On;
    bool sunken  = flags & Style_Sunken;
    bool enabled = flags & Style_Enabled;
    bool horiz   = flags & Style_Horizontal;
    int  x, y, w, h, x2, y2;
    PrimitiveElement arrow;
    QPointArray parray;
    QColorGroup framegroup;
    rect.rect(&x, &y, &w, &h);

    // if no extended colors are set, default from the current ColorGroup
    QColor pcolor(primarycolor.isValid() ? primarycolor : group.button().dark(105));
    QColor scolor(selectioncolor.isValid() ? selectioncolor : group.highlight());

    switch(element) {
      case PE_ButtonDefault:
      case PE_ButtonDropDown:
      case PE_ButtonTool:
          drawAlloyPanel(painter, x, y, w, h, group, (down || on),
                        &group.brush(QColorGroup::Midlight));
          break;

      case PE_ButtonCommand: // SIP
          drawAlloyButton(painter, x, y, w, h, group, (down || on),
                         false, true,
                         (flags & Style_MouseOver));
          break;

      case PE_ButtonBevel:
          drawAlloyBevel(painter, x,y,w,h, group, (down || on), false, false);
          break;

      case PE_HeaderSection: {
          // also kicker taskbar buttons
          bool horizontal = true;
          if (painter && (painter->device()->devType()==QInternal::Widget)) {
              QHeader* header = dynamic_cast<QHeader*>(painter->device());
              if (header) {
                  x--, y--, w++, h++;
                  horizontal = header->orientation() == Horizontal;
              }

          }
          drawAlloyBevel(painter, x, y, w, h, group, (down || on),
                        !horizontal, false);
          break;
      }
      case PE_HeaderArrow:
          if ( flags & Style_Up )
              drawPrimitive( PE_ArrowUp, painter, rect, group, Style_Enabled );
          else
              drawPrimitive( PE_ArrowDown, painter, rect, group, Style_Enabled );
          break;

      case PE_FocusRect: // OK
        painter->setPen(group.highlight().dark(120));
        painter->drawRect(rect);

        break;

      case PE_ScrollBarSlider: // OK
          if (flags & Style_Horizontal) { // horizontal
              drawAlloyGradient(painter, QRect(x+1, y+1, w-2, h-2), pcolor,
                    false, 0, 0, w-2, h-2, down);
              painter->setPen(pcolor.dark(130));
              if (w>60){
                      painter->drawLine(x+w/2-6, y+4, x+w/2-6, h-6);
                      painter->drawLine(x+w/2-3, y+4, x+w/2-3, h-6);
                      painter->drawLine(x+w/2, y+4, x+w/2, h-6);
                      painter->drawLine(x+w/2+3, y+4, x+w/2+3, h-6);
                      painter->drawLine(x+w/2+6, y+4, x+w/2+6, h-6);
                      painter->setPen(pcolor.light(120));
                      painter->drawLine(x+w/2-5, y+5, x+w/2-5, h-5);
                      painter->drawLine(x+w/2-2, y+5, x+w/2-2, h-5);
                      painter->drawLine(x+w/2+1, y+5, x+w/2+1, h-5);
                      painter->drawLine(x+w/2+4, y+5, x+w/2+4, h-5);
                      painter->drawLine(x+w/2+7, y+5, x+w/2+7, h-5);
              } else if (w>40){
                      painter->drawLine(x+w/2-3, y+4, x+w/2-3, h-6);
                      painter->drawLine(x+w/2, y+4, x+w/2, h-6);
                      painter->drawLine(x+w/2+3, y+4, x+w/2+3, h-6);
                      painter->setPen(pcolor.light(120));
                      painter->drawLine(x+w/2-2, y+5, x+w/2-2, h-5);
                      painter->drawLine(x+w/2+1, y+5, x+w/2+1, h-5);
                      painter->drawLine(x+w/2+4, y+5, x+w/2+4, h-5);
              } else if (w>20) {
                      painter->drawLine(x+w/2-1, y+4, x+w/2-1, h-6);
                      painter->drawLine(x+w/2+1, y+4, x+w/2+1, h-6);
                      painter->setPen(pcolor.light(120));
                      painter->drawLine(x+w/2, y+4, x+w/2, h-5);
                      painter->drawLine(x+w/2+2, y+4, x+w/2+2, h-5);
              }
          } else { // vertical
              drawAlloyGradient(painter, QRect(x+1, y+1, w-2, h-2), pcolor,
                    true, 0, 0, w-2, h-2, down);
              painter->setPen(pcolor.dark(130));
              if (h>60){
                      painter->drawLine(x+4, y+h/2-6, w-6, y+h/2-6);
                      painter->drawLine(x+4, y+h/2-3, w-6, y+h/2-3);
                      painter->drawLine(x+4, y+h/2, w-6, y+h/2);
                      painter->drawLine(x+4, y+h/2+3, w-6, y+h/2+3);
                      painter->drawLine(x+4, y+h/2+6, w-6, y+h/2+6);
                      painter->setPen(pcolor.light(120));
                      painter->drawLine(x+5, y+h/2-5, w-5, y+h/2-5);
                      painter->drawLine(x+5, y+h/2-2, w-5, y+h/2-2);
                      painter->drawLine(x+5, y+h/2+1, w-5, y+h/2+1);
                      painter->drawLine(x+5, y+h/2+4, w-5, y+h/2+4);
                      painter->drawLine(x+5, y+h/2+7, w-5, y+h/2+7);
              } else if (h>40){
                      painter->drawLine(x+4, y+h/2-3, w-6, y+h/2-3);
                      painter->drawLine(x+4, y+h/2, w-6, y+h/2);
                      painter->drawLine(x+4, y+h/2+3, w-6, y+h/2+3);
                      painter->setPen(pcolor.light(120));
                      painter->drawLine(x+5, y+h/2-2, w-5, y+h/2-2);
                      painter->drawLine(x+5, y+h/2+1, w-5, y+h/2+1);
                      painter->drawLine(x+5, y+h/2+4, w-5, y+h/2+4);
              } else if (h>20) {
                      painter->drawLine(x+4, y+h/2-1, w-6, y+h/2-1);
                      painter->drawLine(x+4, y+h/2+1, w-6, y+h/2+1);
                      painter->setPen(pcolor.light(120));
                      painter->drawLine(x+4, y+h/2, w-5, y+h/2);
                      painter->drawLine(x+4, y+h/2+2, w-5, y+h/2+2);
              }
          }
          drawAlloyRect(painter, x, y, w, h, pcolor.dark(180));

          break;

      case PE_ScrollBarAddPage: // OK
      case PE_ScrollBarSubPage:
          x2 = rect.right();
          y2 = rect.bottom();

          if (h) { // has a height, thus visible
              painter->fillRect(rect, group.button());
              painter->setPen(group.mid());
              if (flags & Style_Horizontal) { // vertical
                  painter->drawLine(x, y, x2, y);
                  painter->drawLine(x, y2, x2, y2);
              } else { // horizontal
                  painter->drawLine(x, y, x, y2);
                  painter->drawLine(x2, y, x2, y2);
              }
          }
          break;

      case PE_ScrollBarAddLine: // OK
      case PE_ScrollBarSubLine:
          drawAlloyGradient(painter, rect,
                            group.button(), horiz,
                            0, 0, w, h, down);
          drawAlloyRect(painter, x, y, w, h, group.dark());

          arrow = ((flags & Style_Horizontal) ?
                   ((element == PE_ScrollBarAddLine) ?
                        PE_ArrowRight : PE_ArrowLeft) :
                   ((element == PE_ScrollBarAddLine) ?
                        PE_ArrowDown : PE_ArrowUp));

          drawPrimitive(arrow, painter, rect, group, flags);
          break;

      case PE_Indicator: // SIP
          x2 = rect.right();
          y2 = rect.bottom();

          painter->setPen(group.dark());
          painter->drawRect(x, y, w, h);

          drawAlloyGradient(painter, QRect(x+1, y+1, w-2, h-2),
                               group.midlight(), false, 0, 0, w-2, h-2, false);
          if (on) {
              painter->setPen(scolor.dark(140));
              painter->drawLine(x+2, y+4, x+4, y+6);
              painter->drawLine(x+5, y+5, x+8, y+2);
              painter->drawLine(x+2, y+6, x+4, y+8);
              painter->drawLine(x+5, y+7, x+9, y+3);

              painter->setPen(scolor.dark(110));
              painter->drawLine(x+2, y+5, x+4, y+7);
              painter->drawLine(x+5, y+6, x+9, y+2);

              painter->setPen(scolor.dark(170));
              painter->drawLine(x+2, y+7, x+4, y+9);
              painter->drawLine(x+5, y+8, x+9, y+4);
              painter->drawPoint(x+9, y+3);

          }
          break;

      case PE_ExclusiveIndicator: // OK
          x2 = rect.right();
          y2 = rect.bottom();

          if (enabled) {
              drawAlloyGradient(painter, QRect(x+1, y+1, w-2, h-2),
                               group.midlight(), false, 0, 0, w-2, h-2, false);
          } else {
              painter->fillRect(QRect(x+1, y+1, w-2, h-2), group.midlight());
          }
          painter->setPen(group.background());
          painter->drawPoint(x+1, y+1);
          painter->drawPoint(x+1, y2-1);
          painter->drawPoint(x2-1, y+1);
          painter->drawPoint(x2-1, y2-1);

          if (on) {
              painter->setPen(scolor);
              kColorBitmaps(painter, group, x, y, NULL, &radiooffmid,
                            NULL, &radiooffdark, &radiooffshadow);
              painter->setPen(scolor.dark(150));
              painter->drawLine(x+8, y+4, x+8, y+8);
              painter->drawLine(x+7, y+8, x+4, y+8);
              painter->drawPoint(x+9, y+6);
              painter->drawPoint(x+6, y+9);
              painter->drawPoint(x+3, y+6);
              painter->drawPoint(x+4, y+7);
              painter->drawPoint(x+7, y+7);

              painter->setPen(scolor.dark(140));
              painter->drawPoint(x+6, y+3);
              painter->drawPoint(x+7, y+3);
              painter->drawPoint(x+7, y+4);
              painter->drawPoint(x+9, y+5);
              painter->drawPoint(x+9, y+7);
              painter->drawPoint(x+7, y+6);
              painter->drawPoint(x+3, y+7);
              painter->drawPoint(x+5, y+7);
              painter->drawPoint(x+6, y+7);
              painter->drawPoint(x+5, y+9);
              painter->drawPoint(x+7, y+9);
              painter->setPen(scolor.dark(120));
              painter->drawPoint(x+5, y+3);
              painter->drawPoint(x+3, y+5);
              painter->drawPoint(x+4, y+5);
              painter->drawPoint(x+4, y+6);
              painter->drawPoint(x+7, y+5);
              painter->drawPoint(x+6, y+6);
              painter->setPen(scolor.dark(100));
              painter->drawLine(x+4, y+4, x+6, y+4);
              painter->drawPoint(x+5, y+6);
              painter->setPen(scolor.dark(80));
              painter->drawLine(x+5, y+5, x+6, y+5);
              painter->setPen(group.mid());
              painter->drawLine(x+3, y+4, x+4, y+3);
              painter->drawLine(x+3, y+8, x+4, y+9);
              painter->drawLine(x+8, y+9, x+9, y+8);
              painter->drawLine(x+8, y+3, x+9, y+4);

          } else {
              kColorBitmaps(painter, group, x, y, NULL, &radiooffmid,
                            NULL, &radiooffdark, &radiooffshadow);
          }
          break;

      case PE_ExclusiveIndicatorMask:
          painter->setPen(Qt::color1);
          painter->drawPixmap(x, y, radiomask);
          break;

      case PE_DockWindowResizeHandle:
      case PE_Splitter:
          x2 = rect.right();
          y2 = rect.bottom();
          if (w > h) { // horizontal
              painter->fillRect(x, y+1, w, h-2, group.background());
              painter->setPen(group.light());
              painter->drawLine(x, y, x2, y);
              painter->setPen(group.mid());
              painter->drawLine(x, y2, x2, y2);
              if (h > 4) { // only draw handle if adequate space
                  for (int k = 3*w/10; k <= 7*w/10; k = k+6) { // hcentre, 0.4*w wide
                      int yy = y + h/2-1; // vcentre
                      painter->setPen(group.mid().dark(105));
                      painter->drawLine(x+k, yy, x+k, yy+1);
                      painter->drawPoint(x+k+1, yy);
                      painter->setPen(group.mid().light(120));
                      painter->drawPoint(x+k+1, yy+1);
                      painter->setPen(group.mid().light(125));
                      painter->drawPoint(x+k+1, yy+2);
                      painter->drawLine(x+k+2, yy+1, x+k+2, yy+2);
                  }
              }
          } else { // vertical
              painter->fillRect(x+1, y, w-2, h, group.background());
              painter->setPen(group.light());
              painter->drawLine(x, y, x, y2);
              painter->setPen(group.mid());
              painter->drawLine(x2, y, x2, y2);
              if (w > 4) { // only draw handle if adequate space
                  for (int k = 3*h/10; k <= 7*h/10; k = k+6) { // vcentre, 0.4*h high
                      int xx = x + w/2-1; // hcentre
                      painter->setPen(group.mid().dark(105));
                      painter->drawLine(xx, y+k, xx+1, y+k);
                      painter->drawPoint(xx, y+k+1);
                      painter->setPen(group.mid().light(120));
                      painter->drawPoint(xx+1, y+k+1);
                      painter->setPen(group.mid().light(125));
                      painter->drawPoint(xx+2, y+k+1);
                      painter->drawLine(xx+1, y+k+2, xx+2, y+k+2);
                  }
              }
          }
          break;

      case PE_Panel: // OK
      case PE_PanelLineEdit:
      case PE_PanelTabWidget:
          drawAlloyPanel(painter, x, y, w, h, group, sunken, NULL);
          break;

      case PE_PanelPopup:
      case PE_WindowFrame:
          drawAlloyPanel(painter, x, y, w, h, group, false, NULL);
          break;

      case PE_StatusBarSection: // OK
          x2 = rect.right();
          y2 = rect.bottom();
          painter->setPen(group.mid());
          painter->drawLine(x, y,  x2-1, y);
          painter->drawLine(x, y+1, x, y2-1);
          painter->setPen(group.midlight());
          painter->drawLine(x, y2, x2, y2);
          painter->drawLine(x2, y, x2, y2-1);
          break;

      case PE_PanelMenuBar: // toolbar // OK
      case PE_PanelDockWindow:
          x2 = rect.right();
          y2 = rect.bottom();
          // adjust rect so we can use bevel
          //x--; y--; w++; h++;
          //drawAlloyBevel(painter, x, y, w, h, group, false, (w < h), true);
          if (w>h) {
                  drawAlloyGradient(painter, rect, group.button(), false,
                                       0, 0, w, h, false, true);
                  painter->setPen(group.light());
                  painter->drawLine(x, y, x2, y);
                  painter->setPen(group.mid());
                  painter->drawLine(x, y2, x2, y2);
          } else {
                  drawAlloyGradient(painter, rect, group.button(), true,
                                       0, 0, w, h, false, true);
                  painter->setPen(group.light());
                  painter->drawLine(x, y, x, y2);
                  painter->setPen(group.mid());
                  painter->drawLine(x2, y, x2, y2);
          }
          break;

      case PE_DockWindowSeparator:
          x2 = rect.right();
          y2 = rect.bottom();
          drawAlloyGradient(painter, rect, group.button(),
                           !(flags & Style_Horizontal),
                           0, 0, -1, -1, false, true);
          if (flags & Style_Horizontal) {
              painter->setPen(group.mid().dark(115));
              painter->drawLine(w/2-1, y+8, w/2-1, y2-8);

              painter->setPen(group.mid().light(105));
              painter->drawPoint(w/2-1, y+7);
              painter->drawPoint(w/2-1, y2-7);

              painter->setPen(group.mid().light(125));
              painter->drawLine(w/2, y+8, w/2, y2-7);
          } else {
              painter->setPen(group.mid().dark(115));
              painter->drawLine(x+8, h/2-1, x2-8, h/2-1);

              painter->setPen(group.mid().light(105));
              painter->drawPoint(x+7, h/2-1);
              painter->drawPoint(x2-7, h/2-1);

              painter->setPen(group.mid().light(125));
              painter->drawLine(x+8, h/2, x2-7, h/2);
          }
          break;

      case PE_ArrowUp: // OK
      case PE_ArrowDown:
      case PE_ArrowLeft:
      case PE_ArrowRight:
      case PE_SpinWidgetDown:
      case PE_SpinWidgetUp:
          switch (element) {
            case PE_SpinWidgetUp:
            case PE_ArrowUp:
                parray.setPoints(3, -3,+1, +3,+1, +0,-2);
                break;
            case PE_SpinWidgetDown:
            case PE_ArrowDown:
                parray.setPoints(3, -3,-1, +3,-1, +0,+2);
                break;
            case PE_ArrowLeft:
                parray.setPoints(3, 1,-3, +1,+3, -2,0);
                break;
            case PE_ArrowRight:
                parray.setPoints(3, -1,-3, -1,+3, +2,0);
                break;
            default:
                return;
          }

          painter->save();
          parray.translate(x+w/2, y+h/2);
          painter->setPen(enabled ?
                          group.buttonText() : group.mid());
          painter->setBrush(enabled ?
                            group.buttonText() : group.mid());
          painter->drawPolygon(parray);
          painter->restore();
          break;

      default:
          KStyle::drawPrimitive(element, painter, rect, group, flags, option);
    }
}

//////////////////////////////////////////////////////////////////////////////
// drawKStylePrimitive()
// ---------------------
// Draw the element

void Alloy::drawKStylePrimitive(KStylePrimitive element,
                                    QPainter *painter,
                                    const QWidget *widget,
                                    const QRect &rect,
                                    const QColorGroup &group,
                                    SFlags flags,
                                    const QStyleOption &option) const
{
    int x, y, w, h, x2, y2;
    const QSlider* slider;

    rect.rect(&x, &y, &w, &h);
    x2 = rect.right();
    y2 = rect.bottom();

    // if no extended colors are set, default from the current ColorGroup
    QColor scolor(selectioncolor.isValid() ? selectioncolor : group.highlight());

    switch (element) {
      case KPE_ToolBarHandle:
          if (flags & Style_Horizontal) { // vertical dots
              drawAlloyGradient(painter, rect, group.button(), false,
                               0, 0, -1, -1, false, true);
              for (int k = (h%6+3)/2; k <= h-4; k = k+6) {
                  painter->setPen(group.mid().dark(105));
                  painter->drawLine(x+2, y+k, x+3, y+k);
                  painter->drawPoint(x+2, y+k+1);
                  painter->setPen(group.mid().light(120));
                  painter->drawPoint(x+3, y+k+1);
                  painter->setPen(group.mid().light(125));
                  painter->drawPoint(x+4, y+k+1);
                  painter->drawLine(x+3, y+k+2, x+4, y+k+2);
              }
              for (int k = (h%6+3)/2+3; k <= h-7; k = k+6) {
                  painter->setPen(group.mid().dark(105));
                  painter->drawLine(x+5, y+k, x+6, y+k);
                  painter->drawPoint(x+5, y+k+1);
                  painter->setPen(group.mid().light(120));
                  painter->drawPoint(x+6, y+k+1);
                  painter->setPen(group.mid().light(125));
                  painter->drawPoint(x+7, y+k+1);
                  painter->drawLine(x+6, y+k+2, x+7, y+k+2);
              }
          } else { // horizontal dots
              drawAlloyGradient(painter, rect, group.button(), true,
                               0, 0, w, h, false, true);
              for (int k = (w%6+3)/2; k <= w-4; k = k+6) {
                  painter->setPen(group.mid().dark(105));
                  painter->drawLine(x+k, y+2, x+k, y+3);
                  painter->drawPoint(x+k+1, y+2);
                  painter->setPen(group.mid().light(120));
                  painter->drawPoint(x+k+1, y+3);
                  painter->setPen(group.mid().light(125));
                  painter->drawPoint(x+k+1, y+4);
                  painter->drawLine(x+k+2, y+3, x+k+2, y+4);
              }
              for (int k = (w%6+3)/2+3; k <= w-7; k = k+6) {
                  painter->setPen(group.mid().dark(105));
                  painter->drawLine(x+k, y+5, x+k, y+6);
                  painter->drawPoint(x+k+1, y+5);
                  painter->setPen(group.mid().light(120));
                  painter->drawPoint(x+k+1, y+6);
                  painter->setPen(group.mid().light(125));
                  painter->drawPoint(x+k+1, y+7);
                  painter->drawLine(x+k+2, y+6, x+k+2, y+7);
              }
          }
          break;

      case KPE_GeneralHandle:
          if (flags & Style_Horizontal) { // vertical dots
              painter->fillRect(rect, group.brush(QColorGroup::Background));
              for (int k = (h%6+3)/2; k <= h-3; k = k+6) {
                  painter->setPen(group.mid().dark(105));
                  painter->drawLine(x+3, y+k, x+4, y+k);
                  painter->drawPoint(x+3, y+k+1);
                  painter->setPen(group.mid().light(120));
                  painter->drawPoint(x+4, y+k+1);
                  painter->setPen(group.mid().light(125));
                  painter->drawPoint(x+5, y+k+1);
                  painter->drawLine(x+4, y+k+2, x+5, y+k+2);
              }
          } else { // horizontal dots
              painter->fillRect(rect, group.brush(QColorGroup::Background));
              for (int k = (w%6+3)/2; k <= w-3; k = k+6) {
                  painter->setPen(group.mid().dark(105));
                  painter->drawLine(x+k, y+4, x+k, y+5);
                  painter->drawPoint(x+k+1, y+4);
                  painter->setPen(group.mid().light(120));
                  painter->drawPoint(x+k+1, y+5);
                  painter->setPen(group.mid().light(125));
                  painter->drawPoint(x+k+1, y+6);
                  painter->drawLine(x+k+2, y+5, x+k+2, y+6);
              }
          }
          break;

      case KPE_SliderGroove: // OK
          slider = dynamic_cast<const QSlider*>(widget);
          if (slider) {
              if (slider->orientation() == Horizontal) {
                  y +=  (rect.height() / 2) - 2;
                  h = 5;
                  y2 = y + h - 1;
              } else {
                  x += (rect.width() / 2) - 2;
                  w = 5;
                  x2 = x + w - 1;
              }
          }
          drawAlloyRect(painter, x, y-1, w, h+1, group.mid());
          drawAlloyRect(painter, x, y, w, h, group.dark());
          break;

      case KPE_SliderHandle: // SIP
          drawAlloyGradient(painter, QRect(x+5,y+5,w-10,h-10), scolor,
                 false, 0, 0, w-10, h-10, false);

          drawAlloyRect(painter, x+5, y+5, w-10, h-10, scolor.dark(175));
          break;

      default:
          KStyle::drawKStylePrimitive(element, painter, widget, rect,
                                      group, flags, option);
    }
}

//////////////////////////////////////////////////////////////////////////////
// drawControl()
// -------------
// Draw the control

void Alloy::drawControl(ControlElement element,
                            QPainter *painter,
                            const QWidget *widget,
                            const QRect &rect,
                            const QColorGroup &group,
                            SFlags flags,
                            const QStyleOption &option ) const
{
    int x, y, w, h, x2, y2, dx;
    bool active, enabled, focused, down;
    QMenuItem *mi;
    QIconSet::Mode mode;
    QIconSet::State state;
    QPixmap pixmap;

    // if no extended colors are set, default from the current ColorGroup
    QColor pcolor(primarycolor.isValid() ? primarycolor : group.highlight());

    rect.rect( &x, &y, &w, &h );

    switch (element) {
      case CE_PushButton:
          if (widget == highlightwidget) flags |= Style_MouseOver;
              drawPrimitive(PE_ButtonCommand, painter, rect, group, flags);
          break;

      case CE_PushButtonLabel: {
          const QPushButton* button = dynamic_cast<const QPushButton*>(widget);
          if (!button) {
              KStyle::drawControl(element, painter, widget, rect, group,
                                  flags, option);
              return;
          }
          active = button->isOn() || button->isDown();

          if (active) { // shift contents
              x += pixelMetric(PM_ButtonShiftHorizontal, widget);
              y += pixelMetric(PM_ButtonShiftVertical, widget);
              flags |= Style_Sunken;
          }

          if (button->isMenuButton()) { // draw the indicator
              dx = pixelMetric(PM_MenuButtonIndicator, widget);
              w -= dx;
              drawPrimitive(PE_ArrowDown, painter,
                            QRect(x + w - 2, y + 2, dx, h - 4),
                            group, flags, option );
          }

          if (button->iconSet() && !button->iconSet()->isNull()) { // draw icon
              if (button->isEnabled()) {
                  if (button->hasFocus()) {
                      mode = QIconSet::Active;
                  } else {
                      mode = QIconSet::Normal;
                  }
              } else {
                  mode = QIconSet::Disabled;
              }

              if (button->isToggleButton() && button->isOn()) {
                  state = QIconSet::On;
              } else {
                  state = QIconSet::Off;
              }

              pixmap = button->iconSet()->pixmap(QIconSet::Small, mode, state);
//               painter->drawPixmap(x+4, y+h/2 - pixmap.height() / 2, pixmap);
              if (button->text().isEmpty() && !button->pixmap()) {
                  painter->drawPixmap(x+w/2 - pixmap.width()/2,
                                      y+h/2 - pixmap.height()/2, pixmap);
              } else {
                  painter->drawPixmap(x+4, y+h/2 - pixmap.height()/2, pixmap);
              }
              x += pixmap.width() + 4;
              w -= pixmap.width() + 4;
          }

          if ( active || button->isDefault() ) { // default button
              for(int n=0; n<2; n++) {
                  drawItem(painter, QRect(x+n+1, y+1, w, h),
                           AlignCenter | ShowPrefix,
                           button->colorGroup(),
                           button->isEnabled(),
                           button->pixmap(),
                           button->text(), -1,
                           (button->isEnabled()) ?
                               &button->colorGroup().mid() :
                               &button->colorGroup().midlight());
              }
              for(int n=0; n<2; n++) {
                  drawItem(painter, QRect(x+n, y, w, h),
                           AlignCenter | ShowPrefix,
                           button->colorGroup(),
                           button->isEnabled(),
                           button->pixmap(),
                           button->text(), -1,
                           (button->isEnabled()) ?
                               &button->colorGroup().buttonText() :
                               &button->colorGroup().mid());
              }
          } else { // normal button
              drawItem(painter, QRect(x, y, w, h),
                       AlignCenter | ShowPrefix,
                       button->colorGroup(),
                       button->isEnabled(),
                       button->pixmap(),
                       button->text(), -1,
                       (button->isEnabled()) ?
                           &button->colorGroup().buttonText() :
                           &button->colorGroup().mid());
          }

          if (flags & Style_HasFocus) { // draw focus
              drawPrimitive(PE_FocusRect, painter,
                  QStyle::visualRect(subRect(SR_PushButtonFocusRect, widget),
                                     widget), group, flags);
          }
          break;
      }

      case CE_MenuBarItem: {
          const QMenuBar *mbar = dynamic_cast<const QMenuBar*>(widget);
          if (!mbar) {
              KStyle::drawControl(element, painter, widget, rect, group,
                                  flags, option);
              return;
          }
          mi = option.menuItem();
          QRect prect = mbar->rect();

          active  = (flags & Style_Active);
          focused = (flags & Style_HasFocus);
          down = (flags & Style_Down);

          if (active && focused && down) {
              drawAlloyRect(painter, x, y, w, h,
                            group.highlight().dark(160));
              painter->fillRect(QRect(x+1, y+1, w-2, h-2), group.highlight());
          } else {
              drawAlloyGradient(painter, rect, group.button(),
                               false, x, y,
                               prect.width()-2, prect.height()-2, false, true);
          }
          QColor color((active && focused && down) ? group.highlightedText() : group.buttonText());
          drawItem(painter, rect,
                   AlignCenter | AlignVCenter |
                   DontClip | ShowPrefix | SingleLine,
                   group, flags & Style_Enabled,
                   mi->pixmap(), mi->text(), -1, &color);
          break;
      }

      case CE_PopupMenuItem: {// OK
          const QPopupMenu *popup = dynamic_cast<const QPopupMenu*>(widget);
          if (!popup) {
              KStyle::drawControl(element, painter, widget, rect, group,
                                  flags, option);
              return;
          }

          mi = option.menuItem();
          if (!mi) {
              painter->fillRect(rect, group.button());
              break;
          }

          int tabwidth   = option.tabWidth();
          int checkwidth = option.maxIconWidth();
          bool checkable = popup->isCheckable();
          bool etchtext  = styleHint(SH_EtchDisabledText);
          bool reverse   = QApplication::reverseLayout();
          active         = flags & Style_Active;
          enabled        = mi->isEnabled();
          QRect vrect;

          if (checkable) checkwidth = QMAX(checkwidth, 20);

          if (mi->isSeparator()) { // separator
              painter->setPen(group.mid());
              painter->drawLine(x+1, y, x+w-2, y);
              painter->setPen(group.mid().light(110));
              painter->drawPoint(x+1, y);
              painter->drawPoint(x+w-2, y);
              break;
          }

          // draw background
          if (active && enabled) {
              drawAlloyRect(painter, x+2, y+1,
                      w-4, h-2,
                      group.highlight().dark(160));
              painter->fillRect(QRect(x+3, y+2, w-6, h-4), group.highlight());

          } else if (widget->erasePixmap() &&
                     !widget->erasePixmap()->isNull()) {
              painter->drawPixmap(x, y, *widget->erasePixmap(), x, y, w, h);
          } else {
              painter->fillRect(rect, group.button());
          }

          if (mi->iconSet() && !mi->isChecked()) { // draw icon
              vrect = visualRect(QRect(x, y, checkwidth, h), rect);

              if (active)
                  mode = enabled ? QIconSet::Active : QIconSet::Disabled;
              else
                  mode = enabled ? QIconSet::Normal : QIconSet::Disabled;

              pixmap = mi->iconSet()->pixmap(QIconSet::Small, mode);
              QRect pmrect(0, 0, pixmap.width(), pixmap.height());
              pmrect.moveCenter(vrect.center());
              painter->drawPixmap(pmrect.topLeft(), pixmap);
          }

          if (mi->isChecked()) { // draw check
              int cx = reverse ? x+w - checkwidth : x;
              drawPrimitive(PE_CheckMark, painter,
                            QRect(cx + ITEMFRAME, y + ITEMFRAME,
                                  checkwidth - ITEMFRAME*2, h - ITEMFRAME*2),
                            group, Style_Default |
                            (active ? Style_Enabled : Style_On));
          }

          // draw text
          int xm = ITEMFRAME + checkwidth + ITEMHMARGIN;
          int xp = reverse ?
              x + tabwidth + RIGHTBORDER + ITEMHMARGIN + ITEMFRAME - 1 :
              x + xm;
          int offset = reverse ? -1 : 1;
          int tw = w - xm - tabwidth - ARROWMARGIN - ITEMHMARGIN * 3
              - ITEMFRAME + 1;

          if (enabled)
              painter->setPen(active ? group.highlightedText() : group.buttonText());
          else
              painter->setPen(group.mid());

          if (mi->custom()) { // draws own label
              painter->save();
              if (etchtext && !enabled) {
                  painter->setPen(group.light());
                  mi->custom()->paint(painter, group, active, enabled,
                                      xp+offset, y+ITEMVMARGIN+1,
                                      tw, h-2*ITEMVMARGIN);
                  painter->setPen(group.mid());
              }
              mi->custom()->paint(painter, group, active, enabled,
                                  xp, y+ITEMVMARGIN, tw, h-2*ITEMVMARGIN);
              painter->restore();
          }
          else { // draw label
              QString text = mi->text();

              if (!text.isNull()) {
                  int t = text.find('\t');
                  int tflags = AlignVCenter | DontClip |
                      ShowPrefix |  SingleLine |
                      (reverse ? AlignRight : AlignLeft);

                  if (t >= 0) {
                      int tabx = reverse ?
                          x + RIGHTBORDER + ITEMHMARGIN + ITEMFRAME :
                          x + w - tabwidth - RIGHTBORDER - ITEMHMARGIN
                            - ITEMFRAME;

                       // draw right label (accerator)
                      if (etchtext && !enabled) { // etched
                          painter->setPen(group.light());
                          painter->drawText(tabx+offset, y+ITEMVMARGIN+1,
                                            tabwidth, h-2*ITEMVMARGIN,
                                            tflags, text.mid(t+1));
                          painter->setPen(group.mid());
                      }
                      painter->drawText(tabx, y+ITEMVMARGIN,
                                        tabwidth, h-2*ITEMVMARGIN,
                                        tflags, text.mid(t+1));
                      text = text.left(t);
                  }

                  // draw left label
                  if (etchtext && !enabled) { // etched
                      painter->setPen(group.light());
                      painter->drawText(xp+offset, y+ITEMVMARGIN+1,
                                        tw, h-2*ITEMVMARGIN,
                                        tflags, text, t);
                      painter->setPen(group.mid());
                  }
                  painter->drawText(xp, y+ITEMVMARGIN,
                                    tw, h-2*ITEMVMARGIN,
                                    tflags, text, t);
              }
              else if (mi->pixmap()) { // pixmap as label
                  pixmap = *mi->pixmap();
                  if (pixmap.depth() == 1)
                      painter->setBackgroundMode(OpaqueMode);

                  dx = ((w - pixmap.width()) / 2) + ((w - pixmap.width()) % 2);
                  painter->drawPixmap(x+dx, y+ITEMFRAME, pixmap);

                  if (pixmap.depth() == 1)
                      painter->setBackgroundMode(TransparentMode);
              }
          }

          if (mi->popup()) { // draw submenu arrow
              PrimitiveElement arrow = reverse ? PE_ArrowLeft : PE_ArrowRight;
              int dim = (h-2*ITEMFRAME) / 2;
              vrect = visualRect(QRect(x + w - ARROWMARGIN - ITEMFRAME - dim,
                                       y + h / 2 - dim / 2, dim, dim), rect);
              drawPrimitive(arrow, painter, vrect, group,
                            enabled ? Style_Enabled : Style_Default);
          }
          break;
      }

      case CE_TabBarTab: {
          const QTabBar* tabbar = dynamic_cast<const QTabBar*>(widget);
          if (!tabbar) {
              KStyle::drawControl(element, painter, widget, rect, group,
                                  flags, option);
              return;
          }

          QTabBar::Shape tshape = tabbar->shape();
          if(tshape != QTabBar::RoundedAbove) {
              KStyle::drawControl(element, painter, widget, rect, group,
                                  flags, option);
              return;
          }

          bool selected = flags & Style_Selected;
          x2 = rect.right();
          y2 = rect.bottom();

          drawAlloyButton(painter, x, y, rect.width()-1, rect.height(),
                          group, false, true, !selected, false);

          if (!selected) { // finish off bottom

              painter->setPen(group.dark());
              painter->drawPoint(x2-3,y2-1);

          } else {

              painter->setPen(group.background());
              painter->drawLine(x+2,y2-1,x2-4,y2-1);
          }
          if (x==0) {
              painter->setPen(group.dark());
              painter->drawLine(x, y2-1, x, y2);
          }


          break;
      }

      case CE_ProgressBarGroove: { // OK

              drawAlloyGradient(painter, QRect(x+3, y+3, w-5, h-5),
                      group.background().light(105),
                      false, 0, 0, w-5, h+10, true);
              drawAlloyRect(painter, x+1, y+1, w-2, h-2,
                      group.mid());
              drawAlloyRect(painter, x+2, y+2, w-3, h-3,
                      group.dark());

          break;
      }

      case CE_ProgressBarContents: { // OK
          const QProgressBar* pbar = dynamic_cast<const QProgressBar*>(widget);
          if (!pbar) {
              KStyle::drawControl(element, painter, widget, rect, group,
                                  flags, option);
              return;
          }
          QRect subrect = subRect(SR_ProgressBarContents, widget);
          subrect.rect(&x, &y, &w, &h);
          float progress = 1.0;

          if (pbar->totalSteps())
              progress = (float)pbar->progress() / pbar->totalSteps();
          if (progress < 0) progress = 0;
          dx = (int)(w * progress);
          if (dx < 4) return; // don't draw if no bevel

          if (QApplication::reverseLayout()) x += w - dx;
              drawAlloyGradient(painter, QRect(x+3, y+3, dx-4, h-5),
                 pcolor, false, 0, 0, w-6, h-5, false);

          drawAlloyRect(painter, x+2, y+2, dx-3, h-3, pcolor.dark(175));
          break;
      }

      case CE_ProgressBarLabel: { // OK
          const QProgressBar* pbar = dynamic_cast<const QProgressBar*>(widget);
          if (!pbar) {
              KStyle::drawControl(element, painter, widget, rect, group,
                                  flags, option);
              return;
          }
          enabled = pbar->isEnabled();
          if (!enabled) {
              painter->setPen(group.light());
              painter->drawText(x+1, y+1, w, h,
                                AlignCenter | SingleLine,
                                pbar->progressString());
          }
          painter->setPen(enabled ? group.text() : group.mid());
          painter->drawText(rect, AlignCenter | SingleLine,
                            pbar->progressString());
          break;
      }

      default:
          KStyle::drawControl(element, painter, widget, rect, group,
                              flags, option);
    }
}

//////////////////////////////////////////////////////////////////////////////
// drawControlMask()
// -----------------
// Draw a bitmask for the element

void Alloy::drawControlMask(ControlElement element,
                                QPainter *painter,
                                const QWidget *widget,
                                const QRect &rect,
                                const QStyleOption &option) const
{
    switch (element) {
      case CE_PushButton: {
          painter->fillRect(rect, Qt::color1 );
          painter->setPen(Qt::color0);
          break;
      }

      default:
          KStyle::drawControlMask(element, painter, widget, rect, option);
    }
}

//////////////////////////////////////////////////////////////////////////////
// drawComplexControl()
// --------------------
// Draw a complex control

void Alloy::drawComplexControl(ComplexControl control,
                                   QPainter *painter,
                                   const QWidget *widget,
                                   const QRect &rect,
                                   const QColorGroup &group,
                                   SFlags flags,
                                   SCFlags controls,
                                   SCFlags active,
                                   const QStyleOption &option) const
{
    bool down = flags & Style_Down;
    bool on = flags & Style_On;
    bool raised = flags & Style_Raised;
    bool sunken;
    QRect subrect;
    int x, y, w, h, x2, y2;
    rect.rect(&x, &y, &w, &h);

    switch (control) {
      case CC_ComboBox: { // OK
          const QComboBox * combo = dynamic_cast<const QComboBox*>(widget);
          if (!combo) {
              KStyle::drawComplexControl(control, painter, widget, rect, group,
                                         flags, controls, active, option);
              return;
          }

          painter->save();
          sunken = (active == SC_ComboBoxArrow);

          if (controls & SC_ComboBoxEditField) { // draw edit box
              //drawAlloyRect(painter, x, y, w, h, group.dark());
              if (combo->editable()) { // editable box
                  painter->fillRect(rect, group.base());
              } else { // non editable box
                  painter->fillRect(rect, group.button());
                  if (combo->hasFocus()) {
                      subrect = QStyle::visualRect(subRect(SR_ComboBoxFocusRect,
                                                           combo), widget);
                      drawPrimitive(PE_FocusRect, painter, subrect, group,
                                    Style_FocusAtBorder,
                                    QStyleOption(group.highlight()));
                  }
              }
              drawAlloyRect(painter, x, y, w, h, group.mid());
              drawAlloyRect(painter, x+1, y+1, w-1, h-1, group.dark());

              // prepares subrect for the arrow box. IMPORTANT!
              subrect = QStyle::visualRect(querySubControlMetrics(CC_ComboBox, widget,
                                                                  SC_ComboBoxArrow), widget);
              subrect.setBottom(rect.bottom()-2);
              subrect.setTop(rect.top()+3);
          }
          if (controls & SC_ComboBoxArrow) { // draw arrow box
              drawAlloyGradient(painter, subrect, group.button(), false);
              drawAlloyRect(painter, subrect.left(), subrect.top(),
                            subrect.width(), subrect.height(), group.dark());
              drawPrimitive(PE_ArrowDown, painter, subrect, group, flags);
          }

          painter->restore();
          break;
      }

      case CC_SpinWidget: { // OK
          PrimitiveElement element;
          const QSpinWidget *spin = dynamic_cast<const QSpinWidget*>(widget);
          if (!spin) {
              KStyle::drawComplexControl(control, painter, widget, rect, group,
                                         flags, controls, active, option);
              return;
          }

          // draw frame
          subrect = querySubControlMetrics(CC_SpinWidget, widget,
                                           SC_SpinWidgetFrame, option);
          if ((controls & SC_SpinWidgetFrame) && subrect.isValid()) {
              drawAlloyPanel(painter, subrect.x(), subrect.y(),
                            subrect.width(), subrect.height(),
                            group, true, NULL);
          }

          flags |= Style_Enabled; // since Qt never sets this for spin arrows

          // draw up arrow
          subrect = spin->upRect();
          subrect.addCoords(1, 0, 0, 0);
          if ((controls & SC_SpinWidgetUp) && subrect.isValid()) {
              sunken = (active == SC_SpinWidgetUp);
              if (spin->buttonSymbols() == QSpinWidget::PlusMinus)
                  element = PE_SpinWidgetPlus;
              else
                  element = PE_SpinWidgetUp;

              drawAlloyGradient(painter, subrect, group.button(), false);
              painter->setPen(sunken ? group.mid() : group.light());
              painter->drawLine(subrect.x(), subrect.y(),
                                subrect.right(), subrect.y());
              painter->drawLine(subrect.x(), subrect.y()+1,
                                subrect.x(),  subrect.bottom());

              painter->setPen(sunken ? group.light() : group.mid());
              painter->drawLine(subrect.x(),  subrect.bottom(),
                                subrect.right(), subrect.bottom());
              painter->drawLine(subrect.right(), subrect.y(),
                                subrect.right(), subrect.bottom()-1);

              painter->setPen(group.dark());
              painter->drawLine(subrect.x()-1, subrect.y(),
                                subrect.x()-1, subrect.bottom());

              drawPrimitive(element, painter, subrect, group, flags |
                            ((active == SC_SpinWidgetUp) ?
                             Style_On | Style_Sunken : Style_Raised));
          }

          // draw down button
          subrect = spin->downRect();
          subrect.addCoords(1, 0, 0, 0);
          if (rect.height() % 2) { // adjust for odd height
              subrect.addCoords(0, 1, 0, 1);
              painter->setPen(group.dark());
              painter->drawPoint(subrect.x()-1, subrect.y()-1);
          }
          if ((controls & SC_SpinWidgetDown) && subrect.isValid()) {
              sunken = (active == SC_SpinWidgetDown);
              if (spin->buttonSymbols() == QSpinWidget::PlusMinus)
                  element = PE_SpinWidgetMinus;
              else
                  element = PE_SpinWidgetDown;


              drawAlloyGradient(painter, subrect, group.button(), false);
              painter->setPen(sunken ? group.mid() : group.light());
              painter->drawLine(subrect.x(), subrect.y(),
                                subrect.right(), subrect.y());
              painter->drawLine(subrect.x(), subrect.y()+1,
                                subrect.x(),  subrect.bottom());

              painter->setPen(sunken ? group.light() : group.mid());
              painter->drawLine(subrect.x(),  subrect.bottom(),
                                subrect.right(), subrect.bottom());
              painter->drawLine(subrect.right(), subrect.y(),
                                subrect.right(), subrect.bottom()-1);

              painter->setPen(group.dark());
              painter->drawLine(subrect.x()-1, subrect.y(),
                                subrect.x()-1, subrect.bottom());

              drawPrimitive(element, painter, subrect, group, flags |
                            ((active == SC_SpinWidgetDown) ?
                             Style_On | Style_Sunken : Style_Raised));
          }

          break;
      }

      case CC_ToolButton: { // OK
          QToolBar *toolbar = 0;
          QWidget *extension = 0;
          QPointArray pa;
          bool horiz = true;      // orientation of toolbar
          bool floating = false;  // whether toolbar is floating
          const QToolButton *btn = dynamic_cast<const QToolButton*>(widget);
          if (!btn) {
              KStyle::drawComplexControl(control, painter, widget, rect, group,
                                         flags, controls, active, option);
              return;
          }

          x2 = rect.right();
          y2 = rect.bottom();

          if (btn->parent() && (btn->parent()->inherits("QToolBar") ||
                                btn->parent()->inherits("QToolBarExtensionWidget"))) {
              toolbar = dynamic_cast<QToolBar*>(btn->parent());
              if (!toolbar && (extension = dynamic_cast<QWidget*>(btn->parent()))) {
                  toolbar = dynamic_cast<QToolBar*>(extension->parent());
              }
              if (toolbar) {
                  horiz = (toolbar->orientation() == Qt::Horizontal);
                  floating = (toolbar->place() == QDockWindow::OutsideDock);
                  // DlgIndividual is a KGet download dialog
                  if (floating || (toolbar->parent() &&
                                   toolbar->parent()->inherits("DlgIndividual"))) {
                      painter->fillRect(rect, group.button());
                      if (on) painter->fillRect(QRect(x+1, y+1, w-2, h-2),
                                                group.button().dark(110));
                  } else {
                      if (horiz){
                          drawAlloyGradient(painter, QRect(x, y, w, h),
                                           group.button(), !horiz, 0, 0,
                                           toolbar->width()-1, toolbar->height()-1,
                                           false, true);
                          if (on) drawAlloyGradient(painter, QRect(x+1, y+1, w-2, h-2),
                                           group.button().dark(110), !horiz, 0, 0,
                                           toolbar->width()-1, toolbar->height()-1,
                                           false, true);
                      } else {
                          drawAlloyGradient(painter, QRect(x, y, w, h),
                                           group.button(), !horiz, 0, 0,
                                           toolbar->width()-1, toolbar->height()-1,
                                           false, true);
                          if (on) drawAlloyGradient(painter, QRect(x+1, y+1, w-2, h-2),
                                           group.button().dark(110), !horiz, 0, 0,
                                           toolbar->width()-1, toolbar->height()-1,
                                           false, true);
                      }
                  }
              } else {
                  KStyle::drawComplexControl(control, painter, widget, rect,
                                             group, flags, controls, active,
                                             option);
                  return;
              }
          } else { // something strange
              painter->setPen(group.background());
              painter->drawRect(rect);
              painter->fillRect(QRect(x+1, y+1, w-2, h-2),
                                on ? group.button().dark(110) : group.button());
          }

          if (down || on) {
              painter->setPen(group.mid().dark(108));
              painter->drawLine(x, y+3, x, y2-3);
              painter->drawLine(x+3, y, x2-3, y);

              painter->setPen(group.mid().light(105));
              pa.setPoints(4, x,y2-2, x,y+2, x+2,y, x2-2,y);
              painter->drawPoints(pa);

              painter->setPen(group.mid().light(110));
              pa.setPoints(11, x+2,y2, x+1,y2, x+1,y2-1, x,y2-1,
                               x,y+1, x+1,y+1, x+1,y,
                               x2-1,y, x2-1,y+1, x2,y+1, x2,y+2);
              painter->drawPoints(pa);

              painter->setPen(group.light());
              painter->drawLine(x2, y+3, x2, y2-2);
              painter->drawLine(x+3, y2, x2-2, y2);

              painter->setPen(group.light().dark(105));
              pa.setPoints(3, x2,y2-1, x2-1,y2-1, x2-1,y2);
              painter->drawPoints(pa);

          } else if (raised) {
              painter->setPen(group.mid().dark(108));
              painter->drawLine(x+3, y, x2-3, y);
              painter->drawLine(x2, y+3, x2, y2-3);
              painter->drawLine(x+3, y2, x2-3, y2);
              painter->drawLine(x, y+3, x, y2-3);

              painter->setPen(group.mid().light(105));
              pa.setPoints(8, x,y+2, x+2,y, x2-2,y, x2,y+2,
                              x2,y2-2, x2-2,y2, x+2,y2, x,y2-2);
              painter->drawPoints(pa);

              painter->setPen(group.mid().light(110));
              pa.setPoints(12, x+1,y, x+1,y+1, x,y+1,
                               x2-1,y, x2-1,y+1, x2,y+1,
                               x2-1,y2, x2-1,y2-1, x2,y2-1,
                               x+1,y2, x+1,y2-1, x,y2-1);
              painter->drawPoints(pa);
          }
          painter->setPen(group.text());
          break;
      }

      default:
          KStyle::drawComplexControl(control, painter, widget, rect, group,
                                     flags, controls, active, option);
          break;
    }
}

//////////////////////////////////////////////////////////////////////////////
// drawComplexControlMask()
// ------------------------
// Draw a bitmask for the control

void Alloy::drawComplexControlMask(ComplexControl control,
                                       QPainter *painter,
                                       const QWidget *widget,
                                       const QRect &rect,
                                       const QStyleOption &option) const
{
    switch (control) {
      case CC_ComboBox:
      case CC_ToolButton: {
          painter->fillRect(rect, Qt::color1);
          painter->setPen(Qt::color0);
          break;
      }

      default:
          KStyle::drawComplexControlMask(control,painter,widget,rect,option);
    }
}

//////////////////////////////////////////////////////////////////////////////
// pixelMetric()
// -------------
// Get the pixel metric for metric

int Alloy::pixelMetric(PixelMetric metric, const QWidget *widget) const
{
    switch (metric) {
      case PM_ButtonMargin:             // Space betweeen frame and label
          return 6;

      case PM_ButtonDefaultIndicator:   // size of default indicator
          return 0;

      case PM_MenuButtonIndicator:      // Arrow width
          return 7;

      case PM_ExclusiveIndicatorWidth:  // Radiobutton size
      case PM_ExclusiveIndicatorHeight:
          return 13;

      case PM_IndicatorWidth:           // Checkbox size
      case PM_IndicatorHeight:
          return 12;

      case PM_SplitterWidth:
          return 8;

      default:
          return KStyle::pixelMetric(metric, widget);
    }
}

//////////////////////////////////////////////////////////////////////////////
// sizeFromContents()
// ------------------
// Returns the size of widget based on the contentsize

QSize Alloy::sizeFromContents(ContentsType contents,
                                  const QWidget* widget,
                                  const QSize &contentsize,
                                  const QStyleOption &option ) const
{
    int w = contentsize.width();
    int h = contentsize.height();

    switch (contents) {
      case CT_PushButton: {
          const QPushButton* button = dynamic_cast<const QPushButton*>(widget);
          if (!button) {
              return KStyle::sizeFromContents(contents, widget, contentsize,
                                              option);
          }
          int margin = pixelMetric(PM_ButtonMargin, widget) +
              pixelMetric(PM_DefaultFrameWidth, widget) * 2;

          w += margin + 6; // add room for bold font
          h += margin;

          // standard width and heights.
          if (button->isDefault() || button->autoDefault()) {
              if (w < 80 && !button->pixmap())
                  w = 80;
          }
          if (h < 22) h = 22;

          return QSize(w, h);
      }

      case CT_ToolButton: {
          const QToolButton* button = dynamic_cast<const QToolButton*>(widget);
          if (!button) {
              return KStyle::sizeFromContents(contents, widget, contentsize, option);
          }
          return QSize(contentsize.width() + 6, contentsize.height() + 6);
      }

      case CT_PopupMenuItem: {
          if (!widget || option.isDefault()) return contentsize;
          const QPopupMenu *popup = dynamic_cast<const QPopupMenu*>(widget);
          if (!popup) {
              return KStyle::sizeFromContents(contents, widget, contentsize,
                                              option);
          }
          QMenuItem *item = option.menuItem();

          if (item->custom()) {
              w = item->custom()->sizeHint().width();
              h = item->custom()->sizeHint().height();
              if (!item->custom()->fullSpan())
                  h += ITEMVMARGIN*2 + ITEMFRAME*2;
          } else if (item->widget()) {    // a menu item that is a widget
              w = contentsize.width();    // fix by sgehn@gmx.net
              h = contentsize.height();
          } else if (item->isSeparator()) {
              w = h = 2;
          } else {
              if (item->pixmap()) {
                  h = QMAX(h, item->pixmap()->height() + ITEMFRAME*2);
              } else {
                  h = QMAX(h, MINICONSIZE + ITEMFRAME*2);
                  h = QMAX(h, popup->fontMetrics().height()
                           + ITEMVMARGIN*2 + ITEMFRAME*2);
              }
              if (item->iconSet())
                  h = QMAX(h, item->iconSet()->
                           pixmap(QIconSet::Small, QIconSet::Normal).height()
                           + ITEMFRAME*2);
          }

          if (!item->text().isNull() && item->text().find('\t') >= 0)
              w += 12;
          else if (item->popup())
              w += 2 * ARROWMARGIN;

          if (option.maxIconWidth() || popup->isCheckable()) {
               w += QMAX(option.maxIconWidth(),
                         QIconSet::iconSize(QIconSet::Small).width())
                   + ITEMHMARGIN*2;
          }
          w += RIGHTBORDER;

          return QSize(w, h);
      }

      default:
          return KStyle::sizeFromContents(contents, widget, contentsize,
                                          option);
    }
}

//////////////////////////////////////////////////////////////////////////////
// eventFilter()
// -------------
// Grab events we are interested in. Most of this routine is to handle the
// KDE exceptions to the normal styling rules.

bool Alloy::eventFilter(QObject *object, QEvent *event)
{
    if (KStyle::eventFilter(object, event)) return true;

    static QRect wrect, prect;
    bool horiz;
    QWidget *widget, *parent;
    QToolBar *toolbar;
    QPushButton *button;
    QFrame *frame;

    // QPushButton
    if ((button = dynamic_cast<QPushButton*>(object))) {
        if (event->type() == QEvent::Enter) {
            if (button->isEnabled()) {
                highlightwidget = button;
                button->repaint(false);
            }
        } else if (event->type() == QEvent::Leave) {
            if (button == highlightwidget) {
                highlightwidget = 0;
                button->repaint(false);
            }
        }
    }
    // KDE Toolbar Widget
    else if (object->parent() && !qstrcmp(object->name(), TOOLBARWIDGET)) {
        if (event->type() == QEvent::Paint) {
            widget = static_cast<QWidget*>(object);
            parent = static_cast<QWidget*>(object->parent());
            int px = widget->x(), py = widget->y();
            // find the toolbar
            while (parent && parent->parent() &&
                   !qstrcmp(parent->name(), TOOLBARWIDGET)) {
                px += parent->x();
                py += parent->y();
                parent = static_cast<QWidget*>(parent->parent());
            }
            wrect = widget->rect();
            prect = parent->rect();

            QToolBar* toolbar = dynamic_cast<QToolBar*>(parent);
            horiz = (toolbar) ? (toolbar->orientation() == Qt::Horizontal)
                          : prect.height() < prect.width();

            QPainter painter(widget);
            if (horiz) {
                drawAlloyGradient(&painter, wrect, parent->colorGroup().button(),
                        !horiz, px, py,  prect.width(), prect.height(), false, true);
                painter.setPen(parent->colorGroup().mid());
            } else {
                drawAlloyGradient(&painter, wrect, parent->colorGroup().button(),
                        !horiz, px, py,  prect.width(), prect.height(), false, true);
                painter.setPen(parent->colorGroup().mid());
            }
        }
    }
    else if (object->inherits("KToolBarSeparator") && (frame = dynamic_cast<QFrame*>(object))) {
        if (event->type() == QEvent::Paint && !(frame->frameShape() == QFrame::NoFrame)) {
            QPainter painter(frame);
            if (frame->frameShape() == QFrame::VLine) {
                drawPrimitive(PE_DockWindowSeparator, &painter,
                        frame->rect(), frame->colorGroup(), Style_Horizontal);
            } else if (frame->frameShape() == QFrame::HLine) {
                drawPrimitive(PE_DockWindowSeparator, &painter,
                        frame->rect(), frame->colorGroup());
            } else {
                return false;
            }
            return true; // we've already drawn it
        }
    }
    // QToolBarExtensionWidget
    else if ((object->parent()) &&
                (toolbar = dynamic_cast<QToolBar*>(object->parent()))) {
        if (event->type() == QEvent::Paint) {
            widget = static_cast<QWidget*>(object);
            wrect = widget->rect();
            horiz = (toolbar->orientation() == Qt::Horizontal);
            QPainter painter(widget);
            // draw the extension
            if (horiz) {
                drawAlloyGradient(&painter, wrect, toolbar->colorGroup().button(),
                   !horiz, wrect.x(), wrect.y(), prect.width(), prect.height(), false);
                painter.setPen(toolbar->colorGroup().mid());
                painter.drawLine(wrect.x(), wrect.y(),
                                 wrect.x(), wrect.height());
                painter.setPen(toolbar->colorGroup().light());
                painter.drawLine(wrect.x()+1, wrect.y(),
                                 wrect.x()+1, wrect.height());
            } else {
                drawAlloyGradient(&painter, wrect, toolbar->colorGroup().button(),
                    !horiz, wrect.x(), wrect.y(), prect.width(), prect.height(), false);
                painter.setPen(toolbar->colorGroup().light());
                painter.drawLine(wrect.x(), wrect.y(),
                                 wrect.width(), wrect.y());
                painter.setPen(toolbar->colorGroup().mid());
                painter.drawLine(wrect.x(), wrect.y()+1,
                                 wrect.width(), wrect.y()+1);
            }
            return true; // we've already drawn it
        }
    }
    // KFileDialog::toolbar
    else if (!qstrcmp(object->name(), KFILEDIALOGTOOLBAR)) {
        if (event->type() == QEvent::Paint) {

            widget = static_cast<QWidget*>(object);
            wrect = widget->rect();
            QPainter painter(widget);

            drawAlloyGradient(&painter, wrect,
                             widget->colorGroup().button(),
                             (wrect.width() < wrect.height()),
                             0, 0, wrect.width()-1, wrect.height()-1, false, true);
            return true; // we've already drawn it
        }
    }
    return false;
}

//////////////////////////////////////////////////////////////////////////////
// drawAlloyRect()
// ------------------
// Draw rect with antialias corners

void Alloy::drawAlloyRect(QPainter *painter,
        int x, int y, int w, int h,
        const QColor color) const
{
        QPointArray pa;
        int x2 = x+w-1;
        int y2 = y+h-1;

        painter->setPen(color);
        painter->drawRect(x, y, w, h);


        painter->setPen(color.light(115));
        pa.setPoints(8, x+1,y,  x,y+1,  x2-1,y,  x2,y+1,
                        x2,y2-1,  x2-1,y2,  x,y2-1,  x+1,y2);
        painter->drawPoints(pa);

        painter->setPen(color.light(180));
        pa.setPoints(4, x+1,y+1,  x+1,y2-1,  x2-1,y+1,  x2-1,y2-1);
        painter->drawPoints(pa);

        painter->setPen(color.light(175));
        pa.setPoints(4, x,y,  x2,y,  x2,y2,  x,y2);
        painter->drawPoints(pa);

}


//////////////////////////////////////////////////////////////////////////////
// drawAlloyGradient()
// ------------------
// Draw gradient

void Alloy::drawAlloyGradient(QPainter *painter,
                                 const QRect &rect,
                                 QColor color,
                                 bool horizontal,
                                 int px, int py,
                                 int pw, int ph,
                                 bool reverse,
                                 bool light) const
{
    GradientSet *set;

    if (!highcolor || (light && !lightgradientvalue) || (!light && !darkgradientvalue)) {
        painter->fillRect(rect, color);
        return;
    }

    // px, py, pw, ph used for parent-relative pixmaps
    int size;
    if (horizontal)
        size = (pw > 0) ? pw : rect.width();
    else
        size = (ph > 0) ? ph : rect.height();

    if (size > MAXGRADIENTSIZE) { // keep it sensible
        painter->fillRect(rect, color);
    } else {
        if (light) {
            set = lightgradients[color.rgb()][size];
            if (!set) {
                set = new GradientSet(color, size, lightgradientvalue);
                lightgradients[color.rgb()].setAutoDelete(true);
                lightgradients[color.rgb()].insert(size, set);
            }
        } else {
            set = darkgradients[color.rgb()][size];
            if (!set) {
                set = new GradientSet(color, size, darkgradientvalue);
                darkgradients[color.rgb()].setAutoDelete(true);
                darkgradients[color.rgb()].insert(size, set);
            }
        }
        painter->drawTiledPixmap(rect, *set->gradient(horizontal, reverse),
                                 QPoint(px, py));
    }
}

//////////////////////////////////////////////////////////////////////////////
// GradientSet                                                              //
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// GradientSet()
// -------------
// Constructor

GradientSet::GradientSet(const QColor &color, int size, int intensity)
    : color(color), size(size), intensity(100 + intensity)
{
    for (int n=0; n<GradientTypeCount; n++)
        set[n] = 0;
}

//////////////////////////////////////////////////////////////////////////////
// ~GradientSet()
// --------------
// Destructor

GradientSet::~GradientSet()
{
    for (int n=0; n<GradientTypeCount; n++)
        if (set[n]) delete set[n];
}

//////////////////////////////////////////////////////////////////////////////
// gradient()
// ----------
// Return the appropriate gradient pixmap

KPixmap* GradientSet::gradient(bool horizontal, bool reverse)
{
    GradientType type;

    if (horizontal) {
        if (reverse) {
            type = HorizontalReverse;
        } else {
            type = Horizontal;
        }
    } else {
        if (reverse) {
            type = VerticalReverse;
        } else {
            type = Vertical;
        }
    }

    if (set[type]) return (set[type]);

    switch (type) {
      case Horizontal:
          set[type] = new KPixmap();
          set[type]->resize(size, 16);
          KPixmapEffect::gradient(*set[type],
                                  color.light(intensity),
                                  color.dark(intensity),
                                  KPixmapEffect::HorizontalGradient);
          break;

      case HorizontalReverse:
          set[type] = new KPixmap();
          set[type]->resize(size, 16);
          KPixmapEffect::gradient(*set[type],
                                  color.dark(intensity),
                                  color.light(intensity),
                                  KPixmapEffect::HorizontalGradient);
          break;

      case Vertical:
          set[type] = new KPixmap();
          set[type]->resize(16, size);
          KPixmapEffect::gradient(*set[type],
                                  color.light(intensity),
                                  color.dark(intensity),
                                  KPixmapEffect::VerticalGradient);
          break;

      case VerticalReverse:
          set[type] = new KPixmap();
          set[type]->resize(16, size);
          KPixmapEffect::gradient(*set[type],
                                  color.dark(intensity),
                                  color.light(intensity),
                                  KPixmapEffect::VerticalGradient);
          break;

      default:
          break;
    }
    return (set[type]);
}
