/***************************************************************************
 * alloystyle.h ************************************************************
 *                                                                         *
 * Copyright (C) 2003 by Simone Rota <simonerota@operamail.com>            *
 *                                                                         *
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ALLOYSTYLE_H
#define ALLOYSTYLE_H

#include <kstyle.h>
#include <qcolor.h>



class KPixmap;

class Alloy : public KStyle
{
    Q_OBJECT
public:
    Alloy(bool flat=false);
    virtual ~Alloy();

    void polish(QWidget *widget);
    void polish(QPalette &pal);
    void unPolish(QWidget *widget);

    void drawPrimitive(PrimitiveElement element,
            QPainter *painter,
            const QRect &rect,
            const QColorGroup &group,
            SFlags flags = Style_Default,
            const QStyleOption &option = QStyleOption::Default) const;

    void drawKStylePrimitive(KStylePrimitive element,
            QPainter *painter,
            const QWidget *widget,
            const QRect &rect,
            const QColorGroup &group,
            SFlags flags = Style_Default,
            const QStyleOption &option = QStyleOption::Default) const;

    void drawControl(ControlElement element,
            QPainter *painter,
            const QWidget *widget,
            const QRect &rect,
            const QColorGroup &group,
            SFlags flags = Style_Default,
            const QStyleOption &option = QStyleOption::Default) const;

    void drawControlMask(ControlElement element,
            QPainter *painter,
            const QWidget *widget,
            const QRect &rect,
            const QStyleOption &option = QStyleOption::Default) const;

    void drawComplexControl(ComplexControl control,
            QPainter *painter,
            const QWidget *widget,
            const QRect &rect,
            const QColorGroup &group,
            SFlags flags = Style_Default,
            SCFlags controls = SC_All,
            SCFlags active = SC_None,
            const QStyleOption &option = QStyleOption::Default) const;

    void drawComplexControlMask(ComplexControl control,
            QPainter *painter,
            const QWidget *widget,
            const QRect &rect,
            const QStyleOption &option = QStyleOption::Default) const;

    int pixelMetric(PixelMetric metric,
            const QWidget *widget = 0) const;

    QSize sizeFromContents(ContentsType contents,
            const QWidget *widget,
            const QSize &contentsize,
            const QStyleOption& option = QStyleOption::Default) const;

private:
    Alloy(const Alloy &);
    Alloy& operator=(const Alloy &);

    void drawAlloyBevel(QPainter *painter,
            int x, int y, int w, int h,
            const QColorGroup &group,
            bool sunken=false,
            bool horizontal=true,
            bool reverse=false) const;

    void drawAlloyButton(QPainter *painter,
            int x, int y, int w, int h,
            const QColorGroup &group,
            bool sunken=false, bool istab=false, bool filled=true,
            bool highlight=false) const;

    void drawAlloyPanel(QPainter *painter,
            int x, int y, int w, int h,
            const QColorGroup &group,
            bool sunken=false,
            const QBrush *fill=NULL) const;

    void drawAlloyGradient(QPainter *painter,
            const QRect &rect,
            QColor color,
            bool horizontal,
            int px=0, int py=0,
            int pw=-1, int ph=-1,
            bool reverse=false,
            bool light=false) const;

    void drawAlloyRect(QPainter *painter,
            int x, int y, int w, int h,
            const QColor color) const;

    bool eventFilter(QObject *object, QEvent *event);


private:
    QWidget *highlightwidget;
    bool highcolor;
    QColor primarycolor, rollovercolor, selectioncolor;
    int lightgradientvalue, darkgradientvalue;
};

class GradientSet
{
public:
    enum GradientType {
        Horizontal,
        Vertical,
        HorizontalReverse,
        VerticalReverse,
        GradientTypeCount
    };

    GradientSet(const QColor &color, int size, int intensity);
    ~GradientSet();

    KPixmap* gradient(bool horizontal, bool reverse);

private:
    KPixmap *set[GradientTypeCount];
    QColor color;
    int size, intensity;
};

#endif // AlloyStyle_H


